﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using ThinkNet.Domain;
using ThinkNet.Infrastructure.Cache;

namespace ThinkNet.Infrastructure
{
    public class EntityFrameworkRepositoryContext : RepositoryContext, IEntityFrameworkRepositoryContext
    {
        private readonly DbContext _efContext;

        public EntityFrameworkRepositoryContext(DbContext efContext)
        {
            this._efContext = efContext;
        }

        public DbContext Context
        {
            get
            {
                return _efContext;
            }
        }

        public override IDbConnection DbConnection
        {
            get { return _efContext.Database.Connection; }
        }

        private readonly static string cacheRegion = "EntityFramework";
        private static string GetCacheKey(Type type, string id)
        {
            return string.Concat(type.Assembly.FullName, ":", type.FullName, ":", id);
        }

        private static void Evict(IEntity entity)
        {
            string cacheKey = GetCacheKey(entity.GetType(), entity.UniqueId.ToString());

            CacheManager.Remove(cacheRegion, cacheKey);
        }

        public override bool Contains(IEntity entity)
        {
            entity.MustNotNull("entity");
            return Context.Entry(entity).State != EntityState.Detached;
        }

        public override void Detach(IEntity entity)
        {
            entity.MustNotNull("entity");
            _efContext.Entry(entity).State = EntityState.Detached;
        }

        protected override void DoSave(IEntity entity)
        {
            entity.MustNotNull("entity");
            _efContext.Entry(entity).State = EntityState.Added;
        }

        protected override void DoUpdate(IEntity entity)
        {
            entity.MustNotNull("entity");
            _efContext.Entry(entity).State = EntityState.Modified;

            Evict(entity);
        }

        protected override void DoDelete(IEntity entity)
        {
            entity.MustNotNull("entity");
            _efContext.Entry(entity).State = EntityState.Deleted;

            Evict(entity);
        }

        //public override void Persist(IEntity entity)
        //{
        //    entity.MustNotNull("entity");
        //    _efContext.Entry(entity).State = EntityState.Unchanged;

        //    Evict(entity);
        //}

        public override void Refresh(IEntity entity)
        {
            entity.MustNotNull("entity");
            _efContext.Entry(entity).Reload();
        }

        //public override TEntity Get<TEntity, TIdentify>(TIdentify id)
        //{
        //    string cacheKey = GetCacheKey(typeof(TEntity), id.ToString());
        //    object cachedResult = CacheManager.Get(cacheRegion, cacheKey);


        //    if (cachedResult == null) {
        //        cachedResult = this.Load<TEntity>(id);

        //        CacheManager.Put(cacheRegion, cacheKey, cachedResult);
        //    }


        //    return cachedResult as TEntity;
        //}

        public override TEntity Load<TEntity>(object id)
        {
            return _efContext.Set<TEntity>().Find(id);
        }

        protected override void DoCommit()
        {
            try {
                if (!_efContext.ChangeTracker.Entries().Any())
                    return;

                _efContext.SaveChanges();
            }
            catch {
                _efContext.ChangeTracker.Entries()
                    .ToList()
                    .ForEach(entry => entry.State = EntityState.Detached);
                throw;
            }
        }

        protected override void Dispose(bool disposing)
        {
            try {
                base.Dispose(disposing);
            }
            catch (Exception) {
                throw;
            }
            finally {
                if (disposing && _efContext != null) {
                    _efContext.Dispose();
                }
            }
        }


        protected override int DoCount<TEntity>(ICriteria<TEntity> criteria)
        {
            return (criteria ?? new Criteria<TEntity>()).Filtered(_efContext.Set<TEntity>()).Count();
        }

        protected override bool DoExists<TEntity>(ICriteria<TEntity> criteria)
        {
            return this.DoCount(criteria) > 0;
        }

        protected override PagedResult<TEntity> DoQuery<TEntity>(ICriteria<TEntity> criteria, ISort<TEntity> sort, int pageIndex, int pageSize, bool cache, string region)
        {
            IQueryable<TEntity> query = _efContext.Set<TEntity>();


            query = (criteria ?? new Criteria<TEntity>()).Filtered(query);

            int total = 0;
            if (pageIndex >= 0 && pageSize > 0) {
                total = query.Count();
            }

            query = (sort ?? new Sort<TEntity>()).Arranged(query);

            if (pageSize > 0) {
                if (pageIndex > 0) {
                    query = query.Skip(pageIndex * pageSize);
                }
                query = query.Take(pageSize);
            }


            IEnumerable<TEntity> result;

            if (cache) {
                result = query.FromCache(region);
            }
            else {
                result = query.ToList();
            }

            if (pageSize <= 0)
                pageSize = 10;

            return new PagedResult<TEntity>(total, pageSize, pageIndex, result);
        }
    }
}
