﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Text;


using ThinkNet.Infrastructure.Cache;

namespace ThinkNet.Infrastructure
{
    public static class EntityFrameworkExtensions
    {
        internal static LinkedList<T> ToLinkedList<T>(this IEnumerable<T> source)
        {
            return new LinkedList<T>(source);
        }

        internal static string ToConcatenatedString<T>(this IEnumerable<T> source, Func<T, string> selector, string separator)
        {
            var b = new StringBuilder();
            bool needSeparator = false;


            foreach (var item in source) {
                if (needSeparator)
                    b.Append(separator);


                b.Append(selector(item));
                needSeparator = true;
            }


            return b.ToString();
        }




        internal static string GetCacheKey(this IQueryable query)
        {
            var expression = query.Expression;


            // locally evaluate as much of the query as possible
            expression = Evaluator.PartialEval(expression, CanBeEvaluatedLocally);


            // support local collections
            expression = LocalCollectionExpander.Rewrite(expression);


            // use the string representation of the expression for the cache key
            string key = expression.ToString();


            return Utils.MD5(key);
        }

        static Func<Expression, bool> CanBeEvaluatedLocally
        {
            get
            {
                return expression => {
                    // don't evaluate parameters
                    if (expression.NodeType == ExpressionType.Parameter)
                        return false;


                    // can't evaluate queries
                    if (typeof(IQueryable).IsAssignableFrom(expression.Type))
                        return false;


                    return true;
                };
            }
        }



        public static IEnumerable<TEntity> FromCache<TEntity>(this IQueryable<TEntity> query, string cacheRegion = null)
            where TEntity : class
        {
            string cacheKey = query.GetCacheKey();
            if (string.IsNullOrEmpty(cacheRegion)) {
                cacheRegion = "ThinkNet-EFCache";
            }

            object cachedResult = CacheManager.Get(cacheRegion, cacheKey);
            if (cachedResult == null) {
                cachedResult = query.AsNoTracking().ToList();
                CacheManager.Put(cacheRegion, cacheKey, cachedResult);
            }


            return cachedResult as IEnumerable<TEntity>;
        }

    }
}
