﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using MySql.Data.MySqlClient;

namespace ThinkNet.Database
{
    public class MySqlDatabase : Database
    {
        /// <summary>
        /// 通过有效的数据连接字符串构造实例
        /// </summary>
        /// <param name="connectionString"></param>
        public MySqlDatabase(string connectionString)
            : base(MySqlClientFactory.Instance, connectionString)
        { }

        public MySqlDatabase(IDbConnection connection) :
            base(connection)
        {
            if (!(connection is MySqlConnection)) {
                throw new DatabaseException("此数据连接非MySqlConnection");
            }
        }

        protected override string BuildParameterName(string name)
        {
            return string.Concat("?", name);
        }

        protected override void DeriveParameters(DbCommand discoveryCommand)
        {
            if (discoveryCommand != null) {
                MySqlCommandBuilder.DeriveParameters(discoveryCommand as MySqlCommand);
            }
        }

        protected override DbParameter CreateParameter(string paramName, DbType dbType, ParameterDirection direction, int size, object value)
        {
            return CreateMySqlParameter(paramName, (MySqlDbType)dbType, direction, size, value);
        }

        public override DataSet ExecuteDataset(CommandType commandType, string commandText, params IDataParameter[] commandParameters)
        {
            commandText.MustNotEmpty("commandText");

            using (IDbCommand command = CreateCommandByCommandType(commandType, commandText, commandParameters)) {
                try {
                    using (MySqlDataAdapter da = new MySqlDataAdapter()) {
                        //da.MissingSchemaAction = MissingSchemaAction.AddWithKey;
                        da.SelectCommand = command as MySqlCommand;
                        DataSet ds = new DataSet();

                        // Fill the DataSet using default values for DataTable names, etc
                        da.Fill(ds);

                        // Return the dataset
                        return ds;
                    }
                }
                catch (Exception ex) {
                    throw ex;
                }
                finally {
                    command.Parameters.Clear();
                }
            }
        }

        protected override string GetIdentitySql()
        {
            return "SELECT LAST_INSERT_ID()";
        }

        protected override DbColumn[] GetTableColumns(string tableName)
        {
            List<DbColumn> arrField = new List<DbColumn>();

            IDataReader reader = base.ExecuteReader(CommandType.Text,
                string.Format("SHOW COLUMNS FROM `{0}`", tableName), (IDataParameter[])null);
            while (reader.Read()) {
                DbColumn field = new DbColumn();
                field.ColumnName = reader.GetString(0);
                field.DataType = reader.GetString(1);
                field.AllowDBNull = reader.GetString(2).Equals("YES", StringComparison.OrdinalIgnoreCase);
                field.AutoIncrement = reader.GetString(5).Equals("auto_increment", StringComparison.OrdinalIgnoreCase);
                field.Primary = reader.GetString(3).Equals("PRI", StringComparison.OrdinalIgnoreCase);

                arrField.Add(field);
            }
            reader.Close();

            return arrField.ToArray();
        }

        protected override string BuildLikeParameterName(string parameterName)
        {
            return string.Concat("concat('%',", parameterName, ",'%')");
        }

        protected override string PagingSupport(int pageIndex, int pageSize)
        {
            return string.Format("SELECT%DISTINCT%%FIELDS%%TABLE%%JOIN%%WHERE%%GROUP%%HAVING%%ORDER% LIMIT {0},{1}", pageIndex * pageSize, pageSize);
        }

        #region
        /// <summary>
        /// Command对象的传入参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">数据类型</param>
        /// <param name="size">数据长度</param>
        /// <param name="value">值</param>
        /// <returns></returns>
        public static DbParameter CreateInParameter(string paramName, MySqlDbType dbType, int size, object value)
        {
            return CreateMySqlParameter(paramName, dbType, ParameterDirection.Input, size, value);
        }
        /// <summary>
        /// Command对象的传入参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">数据类型</param>
        /// <param name="value">值</param>
        /// <returns></returns>
        public static DbParameter CreateInParameter(string paramName, MySqlDbType dbType, object value)
        {
            return CreateInParameter(paramName, dbType, -1, value);
        }
        /// <summary>
        /// Command对象的传出参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">数据类型</param>
        /// <param name="size">数据长度</param>
        /// <returns></returns>
        public static DbParameter CreateOutParameter(string paramName, MySqlDbType dbType, int size)
        {
            return CreateMySqlParameter(paramName, dbType, ParameterDirection.Output, size, null);
        }
        /// <summary>
        /// Command对象的传出参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">数据类型</param>
        /// <returns></returns>
        public static DbParameter CreateOutParameter(string paramName, MySqlDbType dbType)
        {

            return CreateOutParameter(paramName, dbType, -1);
        }

        /// <summary>
        /// 创建Commond用的DbParameter参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">参数对应的数据类型枚举</param>
        /// <param name="direction">参数类型</param>
        /// <param name="size">大小</param>
        /// <param name="value">值</param>
        /// <returns></returns>
        private static DbParameter CreateMySqlParameter(string paramName, MySqlDbType dbType, ParameterDirection direction, int size, object value)
        {
            MySqlParameter param = new MySqlParameter();
            param.ParameterName = string.Concat("?", paramName);
            param.MySqlDbType = dbType;
            param.Direction = direction;
            if (size > 0)
                param.Size = size;
            if (!(direction == ParameterDirection.Output && value == null))
                param.Value = value;

            return param;
        }
        #endregion
    }
}
