﻿using System.Collections.Generic;
using System.Data;
using System.Linq;
using NHibernate;
using NHibernate.Linq;
using ThinkNet.Domain;
using System;


namespace ThinkNet.Infrastructure
{
    public class NHibernateRepositoryContext : RepositoryContext, INHibernateRepositoryContext
    {
        public NHibernateRepositoryContext()
        { }

        //public NHibernateRepositoryContext(ISession session)
        //{
        //    this._session = session;
        //}

        private ISession _session = null;
        private readonly object sync = new object();

        /// <summary>
        /// 返回NHibernate的一个session
        /// </summary>
        public ISession Session
        {
            get
            {
                if (_session == null) {
                    try {
                        _session = NHibernateSessionFactory.Instance.OpenSession();//ServiceLocator.Current.GetInstance<ISession>();
                        _session.BeginTransaction();
                    }
                    catch { throw; }
                }

                return _session;
            }
        }

        public override IDbConnection DbConnection
        {
            get
            {
                return Session.Connection;
            }
        }

        public override bool Contains(IEntity entity)
        {
            return Session.Contains(entity);
        }

        protected override void DoDelete(IEntity entity)
        {
            Session.Delete(entity);
        }

        protected override void DoSave(IEntity entity)
        {
            Session.Save(entity);
        }

        protected override void DoUpdate(IEntity entity)
        {
            Session.Update(entity);
        }

        public override void Detach(IEntity entity)
        {
            Session.Evict(entity);
        }

        //public override void Persist(IEntity entity)
        //{
        //    Session.Persist(entity);
        //}

        public override void Refresh(IEntity entity)
        {
            Session.Refresh(entity);
        }

        public override TEntity Load<TEntity>(object id)
        {
            return Session.Get<TEntity>(id);
        }

        //public override TEntity Get<TEntity, TIdentify>(TIdentify id)
        //{
        //    return Session.Get<TEntity>(id);
        //}

        protected override void Dispose(bool disposing)
        {
            try {
                base.Dispose(disposing);
            }
            catch (Exception) {
                throw;
            }
            finally {
                if (disposing && _session != null) {
                    if (_session.IsOpen) {
                        _session.Close();
                    }

                    _session.Dispose();
                }
            }
        }

        // 提交事务
        protected override void DoCommit()
        {
            lock (sync) {
                using (ITransaction transaction = Session.Transaction) {
                    try {
                        transaction.Commit();
                    }
                    catch {
                        if (transaction.IsActive)
                            transaction.Rollback();
                        Session.Clear();
                        throw;
                    }
                }
            }
        }

        protected override int DoCount<TEntity>(ICriteria<TEntity> criteria)
        {
            return (criteria ?? new Criteria<TEntity>()).Filtered(Session.Query<TEntity>()).Count();
        }

        protected override bool DoExists<TEntity>(ICriteria<TEntity> criteria)
        {
            return (criteria ?? new Criteria<TEntity>()).Filtered(Session.Query<TEntity>()).Any();
        }

        protected override PagedResult<TEntity> DoQuery<TEntity>(ICriteria<TEntity> criteria, ISort<TEntity> sort, int pageIndex, int pageSize, bool cache, string region)
        {
            IQueryable<TEntity> query = Session.Query<TEntity>();

            query = (criteria ?? new Criteria<TEntity>()).Filtered(query);

            int total = 0;
            if (pageIndex >= 0 && pageSize > 0) {
                total = query.Count();
            }

            query = (sort ?? new Sort<TEntity>()).Arranged(query);

            if (pageSize > 0) {
                if (pageIndex > 0) {
                    query = query.Skip(pageIndex * pageSize);
                }
                query = query.Take(pageSize);
            }

            if (cache) {
                query = query.Cacheable();

                if (!string.IsNullOrEmpty(region)) {
                    query = query.CacheRegion(region);
                }
            }

            IList<TEntity> result = query.ToList();

            if (pageSize <= 0)
                pageSize = 10;

            return new PagedResult<TEntity>(total, pageSize, pageIndex, result);
        }
    }
}
