﻿using System.Reflection;
using System.Text;

using Microsoft.Practices.Unity.InterceptionExtension;
using ThinkNet.Infrastructure.Cache;

namespace ThinkNet.Infrastructure
{
    internal class CachingCallHandler : ICallHandler
    {
        private readonly CachingHandlerAttribute attribute;

        public CachingCallHandler(CachingHandlerAttribute attribute)
        {
            this.attribute = attribute;

            this.Order = attribute.Order;
        }

        #region ICallHandler 成员

        public IMethodReturn Invoke(IMethodInvocation input, GetNextHandlerDelegate getNext)
        {
            string cacheKey = attribute.CacheKey;
            if (string.IsNullOrEmpty(cacheKey)) {
                cacheKey = CreateCacheKey(input);
            }

            string cacheRegion = attribute.CacheRegion;
            if (string.IsNullOrEmpty(cacheRegion)) {
                cacheRegion = "ThinkNet-Common";
            }

            switch (attribute.Method) {
                case CachingMethod.Get:
                    if (TargetMethodReturnsVoid(input)) {
                        return getNext()(input, getNext);
                    }


                    object cachedResult = CacheManager.Get(cacheRegion, cacheKey);

                    if (cachedResult == null) {
                        IMethodReturn realReturn = getNext()(input, getNext);
                        CacheManager.Put(cacheRegion, cacheKey, realReturn.ReturnValue);

                        return realReturn;
                    }
                    else {
                        IMethodReturn cachedReturn = input.CreateMethodReturn(cachedResult, input.Arguments);
                        return cachedReturn;
                    }

                case CachingMethod.Put:
                    IMethodReturn methodReturn = getNext().Invoke(input, getNext);
                    CacheManager.Put(cacheRegion, cacheKey, methodReturn.ReturnValue);

                    return methodReturn;
                case CachingMethod.Remove:
                    foreach (var region in attribute.RelatedAreas) {
                        CacheManager.Evict(region);
                    }
                    return getNext().Invoke(input, getNext);
                default:
                    return getNext().Invoke(input, getNext);
            }
        }

        public int Order
        {
            get;
            set;
        }

        #endregion

        private bool TargetMethodReturnsVoid(IMethodInvocation input)
        {
            MethodInfo targetMethod = input.MethodBase as MethodInfo;
            return targetMethod != null && targetMethod.ReturnType == typeof(void);
        }

        private string CreateCacheKey(IMethodInvocation input)
        {
            StringBuilder sb = new StringBuilder();

            var method = input.MethodBase;

            //sb.AppendFormat("{0}:", method.Name);
            if (method.DeclaringType != null) {
                sb.Append(method.DeclaringType.FullName).Append(":");
                //sb.Append(method.DeclaringType.FullName);
                //sb.Append(':');
            }
            sb.Append(method.Name);

            foreach (object param in input.Inputs) {
                if (param != null) {
                    sb.Append(':').Append(param.ToString());
                }
            }

            return sb.ToString();
        }
    }
}
