﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;

using Microsoft.Practices.Unity;
using Microsoft.Practices.Unity.InterceptionExtension;


namespace ThinkNet.Infrastructure
{
    internal class UnityObjectContainer : IObjectContainer
    {
        internal static LifetimeManager GetLifetime(LifetimeStyle lifestyle)
        {
            switch (lifestyle) {
                case LifetimeStyle.PerThread:
                    return new PerThreadLifetimeManager();
                case LifetimeStyle.Singleton:
                    return new ContainerControlledLifetimeManager();
                case LifetimeStyle.PerRequest:
                    return new PerSessionLifetimeManager();
                default:
                    return new TransientLifetimeManager();
            }
        }


        private readonly IUnityContainer container;

        public UnityObjectContainer()
            : this(new UnityContainer())
        { }

        public UnityObjectContainer(IUnityContainer container)
        {
            this.container = container;
        }

        public IUnityContainer UnityContainer
        {
            get { return this.container; }
        }



        public bool IsRegistered(Type type)
        {
            return container.IsRegistered(type);
        }

        public bool IsRegistered(Type type, string name)
        {
            return container.IsRegistered(type, name);
        }

        public IObjectContainer RegisterInstance(Type type, object instance, LifetimeStyle lifetimeStyle)
        {
            container.RegisterInstance(type, instance, GetLifetime(lifetimeStyle));

            return this;
        }

        public IObjectContainer RegisterInstance(Type type, object instance, string name, LifetimeStyle lifetimeStyle)
        {
            container.RegisterInstance(type, name, instance, GetLifetime(lifetimeStyle));

            return this;
        }

        public IObjectContainer RegisterType(Type type, LifetimeStyle lifetimeStyle)
        {
            var lifetime = GetLifetime(lifetimeStyle);

            if (DefineHandlerAttribute(type)) {
                container.RegisterType(type, lifetime,
                    new Interceptor<VirtualMethodInterceptor>(),
                    new InterceptionBehavior<PolicyInjectionBehavior>());
            }
            else {
                container.RegisterType(type, lifetime);
            }

            return this;
        }

        public IObjectContainer RegisterType(Type type, string name, LifetimeStyle lifetimeStyle)
        {
            var lifetime = GetLifetime(lifetimeStyle);

            if (DefineHandlerAttribute(type)) {
                container.RegisterType(type, name, lifetime,
                    new Interceptor<VirtualMethodInterceptor>(),
                    new InterceptionBehavior<PolicyInjectionBehavior>());
            }
            else {
                container.RegisterType(type, name, lifetime);
            }

            return this;
        }

        public IObjectContainer RegisterType(Type from, Type to, LifetimeStyle lifetimeStyle)
        {
            bool implementHasIntercept = DefineHandlerAttribute(to);
            bool interfaceHasIntercept = DefineHandlerAttribute(from);

            var lifetime = GetLifetime(lifetimeStyle);

            if (implementHasIntercept && interfaceHasIntercept) {
                container.RegisterType(from, to, lifetime,
                    new Interceptor<InterfaceInterceptor>(),
                    new Interceptor<VirtualMethodInterceptor>(),
                    new InterceptionBehavior<PolicyInjectionBehavior>());
            }
            else if (!implementHasIntercept && interfaceHasIntercept) {
                container.RegisterType(from, to, lifetime,
                    new Interceptor<InterfaceInterceptor>(),
                    new InterceptionBehavior<PolicyInjectionBehavior>());
            }
            else if (implementHasIntercept && !interfaceHasIntercept) {
                container.RegisterType(from, to, lifetime,
                     new Interceptor<VirtualMethodInterceptor>(),
                     new InterceptionBehavior<PolicyInjectionBehavior>());
            }
            else {
                container.RegisterType(from, to, lifetime);
            }

            return this;
        }

        public IObjectContainer RegisterType(Type from, Type to, string name, LifetimeStyle lifetimeStyle)
        {
            bool implementHasIntercept = DefineHandlerAttribute(to);
            bool interfaceHasIntercept = DefineHandlerAttribute(from);

            var lifetime = GetLifetime(lifetimeStyle);

            if (implementHasIntercept && interfaceHasIntercept) {
                container.RegisterType(from, to, name, lifetime,
                    new Interceptor<InterfaceInterceptor>(),
                    new Interceptor<VirtualMethodInterceptor>(),
                    new InterceptionBehavior<PolicyInjectionBehavior>());
            }
            else if (!implementHasIntercept && interfaceHasIntercept) {
                container.RegisterType(from, to, name, lifetime,
                    new Interceptor<InterfaceInterceptor>(),
                    new InterceptionBehavior<PolicyInjectionBehavior>());
            }
            else if (implementHasIntercept && !interfaceHasIntercept) {
                container.RegisterType(from, to, name, lifetime,
                     new Interceptor<VirtualMethodInterceptor>(),
                     new InterceptionBehavior<PolicyInjectionBehavior>());
            }
            else {
                container.RegisterType(from, to, name, lifetime);
            }

            return this;
        }

        public object Resolve(Type type)
        {
            if (!container.IsRegistered(type)) 
                return null;

            return container.Resolve(type);
        }

        public object Resolve(Type type, string key)
        {
            if (!container.IsRegistered(type, key))
                return null;

            return container.Resolve(type, key);
        }

        public IEnumerable<object> ResolveAll(Type type)
        {
            return container.ResolveAll(type);
        }

        public void Dispose()
        {
            container.Dispose();
        }


        private static bool DefineHandlerAttribute(Type type)
        {
            return type.GetAttributes<HandlerAttribute>(false).Any()
                //||
                //type.GetMethods(BindingFlags.Public | BindingFlags.NonPublic)
                //.Any(p => p.GetAttributes<HandlerAttribute>(false).Any())
                ||
                type.GetMembers().Any(p => p.GetAttributes<HandlerAttribute>(false).Any());
        }
    }
}
