﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Web;

using ThinkNet.Command;
using ThinkNet.Domain;
using ThinkNet.Event;
using ThinkNet.Infrastructure;
using ThinkNet.Messaging;


namespace ThinkNet
{
    /// <summary>
    /// 系统引导程序
    /// </summary>
    public sealed class Bootstrapper
    {

        private static readonly Bootstrapper _current = new Bootstrapper();

        static Bootstrapper()
        { }

        public static Bootstrapper Current
        {
            get
            {
                return _current;
            }
        }

        private Bootstrapper()
        { }

        private readonly List<IAssemblyInitializer> _plugins = new List<IAssemblyInitializer>();
        private readonly List<Assembly> _assemblies = new List<Assembly>();
        //private readonly List<ICommandProcessor> _commandProcessors = new List<ICommandProcessor>();

        ///// <summary>Get all the command queues.
        ///// </summary>
        //public IEnumerable<ICommandQueue> GetCommandQueues()
        //{
        //    return _commandProcessors.Select(x => x.BindingQueue);
        //}

        /// <summary>
        /// 加载程序集
        /// </summary>
        /// <param name="assemblies"></param>
        /// <returns></returns>
        public Bootstrapper LoadAssemblies(params Assembly[] assemblies)
        {
            _assemblies.Clear();
            _assemblies.AddRange(assemblies);

            return this;
        }

        /// <summary>
        /// 加载程序集
        /// </summary>
        /// <param name="assemblies"></param>
        /// <returns></returns>
        public Bootstrapper LoadAssemblies(params string[] assemblies)
        {
            return this.LoadAssemblies(Array.ConvertAll(assemblies, assembly => Assembly.Load(assembly)));
        }

        /// <summary>
        /// 添加程序初始化的自定义插件
        /// </summary>
        /// <param name="plugin"></param>
        /// <returns></returns>
        public Bootstrapper AddPlugins(params IAssemblyInitializer[] initializers)
        {
            if (initializers != null && initializers.Any())
                _plugins.AddRange(initializers);

            return this;
        }

#if NET35
        public Bootstrapper UseDefaultContainer()
        {
            return this.SetContainer(new TinyContainer());
        }
#endif


        private IObjectContainer _container;
        /// <summary>
        /// 设置IOC容器
        /// </summary>
        /// <param name="container"></param>
        /// <returns></returns>
        public Bootstrapper SetContainer(IObjectContainer container)
        {
            container.MustNotNull("container");
            this._container = container;

            return this;
        }

        public IObjectContainer Container
        {
            get { return this._container; }
        }

        /// <summary>
        /// 类型注册，此方法必须在设置IOC容器后才能使用
        /// </summary>
        /// <param name="lifetime"></param>
        /// <returns></returns>
        public Bootstrapper Register<TService, TImplementer>(LifetimeStyle lifetime)
        {
            _container.RegisterType<TService, TImplementer>(lifetime);

            if (IsAssemblyInitializer(typeof(TImplementer)) && lifetime == LifetimeStyle.Singleton) {
                var instance = ServiceLocator.GetInstance(typeof(TService));
                _plugins.Add((IAssemblyInitializer)instance);
            }

            return this;
        }
        
        /// <summary>
        /// 注册类型的实例
        /// </summary>
        /// <param name="instance"></param>
        /// <param name="lifetime"></param>
        /// <returns></returns>
        public Bootstrapper Register<TService>(TService instance, LifetimeStyle lifetime)
        {
            _container.RegisterInstance<TService>(instance, lifetime);

            if (IsAssemblyInitializer(instance) && lifetime == LifetimeStyle.Singleton) {
                _plugins.Add((IAssemblyInitializer)instance);
            }

            return this;
        }

        /// <summary>
        /// 注册框架默认的组件
        /// </summary>
        /// <returns></returns>
        public Bootstrapper RegisterDefaultComponents()
        {
            Register<ILogger, EmptyLogger>(LifetimeStyle.Singleton);
            Register<IBinarySerializer, DefaultBinarySerializer>(LifetimeStyle.Singleton);
            Register<IStringSerializer, DefaultJsonSerializer>(LifetimeStyle.Singleton);

            Register<IAggregateRootTypeProvider, DefaultAggregateRootTypeProvider>(LifetimeStyle.Singleton);
            Register<IAggregateRootInternalHandlerProvider, DefaultAggregateRootInternalHandlerProvider>(LifetimeStyle.Singleton);
            Register<ICache, MemoryCache>(LifetimeStyle.Singleton);
            Register<IRepository, EventSourcedRepository>(LifetimeStyle.Singleton);
            Register<ISnapshotStore, EmptySnapshotStore>(LifetimeStyle.Singleton);

            Register<IMessageStore, EmptyMessageStore>(LifetimeStyle.Singleton);
            

            Register<IEventStore, EventStoreInMemory>(LifetimeStyle.Singleton);
            Register<IEventHandleInfoStore, EventHandleInfoInMemory>(LifetimeStyle.Singleton);

            return this;
        }

        /// <summary>
        /// 注册标识 <ParticipateInContainerAttribute> 特性标签的组件
        /// </summary>
        /// <returns></returns>
        public Bootstrapper RegisterBusinessComponents()
        {
            _assemblies
                .SelectMany(assembly => assembly.GetTypes().Where(IsComponent))
                .ToList()
                .ForEach(type => {
                    var attribute = type.GetAttribute<ParticipateInContainerAttribute>(false);
                    attribute.Register(_container, type);

                    if (IsAssemblyInitializer(type) && attribute.LifetimeStyle == LifetimeStyle.Singleton) {
                        var instance = _container.Resolve(type);
                        _plugins.Add((IAssemblyInitializer)instance);
                    }
                });

            return this;
        }

        /// <summary>
        /// 创建命令处理器
        /// </summary>
        /// <param name="commandQueueNames"></param>
        /// <param name="workCount"></param>
        /// <returns></returns>
        public Bootstrapper CreateCommandProcessors(IEnumerable<string> commandQueueNames, int workCount = 1)
        {
            Register<ICommandHandlerProvider, DefaultCommandHandlerProvider>(LifetimeStyle.Singleton);
            Register<ICommandContext, DefaultCommandContext>(LifetimeStyle.Transient);
            Register<ICommandQueueRouter, DefaultCommandQueueRouter>(LifetimeStyle.Singleton);
            Register<ICommandAsyncResultManager, DefaultCommandAsyncResultManager>(LifetimeStyle.Singleton);
            Register<ICommandExecutor, DefaultCommandExecutor>(LifetimeStyle.Transient);

            CommandProcessorManager.Instance.CreateDefault(commandQueueNames, workCount);

            return this;
        }

        /// <summary>
        /// 创建事件处理器
        /// </summary>
        /// <param name="eventQueueNames"></param>
        /// <param name="workCount"></param>
        /// <returns></returns>
        public Bootstrapper CreateEventProcessors(IEnumerable<string> eventQueueNames, int workCount = 1)
        {
            Register<IEventHandlerProvider, DefaultEventHandlerProvider>(LifetimeStyle.Singleton);
            Register<IEventSynchronizerProvider, DefaultEventSynchronizerProvider>(LifetimeStyle.Singleton);
            Register<IEventTableNameProvider, DefaultEventTableNameProvider>(LifetimeStyle.Singleton);
            Register<IEventPublisher, DefaultEventPublisher>(LifetimeStyle.Singleton);
            Register<ICommittedEventQueueRouter, DefaultCommittedEventQueueRouter>(LifetimeStyle.Singleton);
            Register<ICommittedEventExecutor, DefaultCommittedEventExecutor>(LifetimeStyle.Transient);

            EventProcessorManager.Instance.CreateDefault(eventQueueNames, workCount);

            return this;
        }


        
        private static bool IsAssemblyInitializer(Type type)
        {
            return type.IsClass && !type.IsAbstract && typeof(IAssemblyInitializer).IsAssignableFrom(type);
        }
        private static bool IsAssemblyInitializer(object instance)
        {
            return instance is IAssemblyInitializer;
        }


        /// <summary>
        /// 启动运行
        /// </summary>
        public void Run()
        {
            if (_assemblies.Count == 0)
                throw new ThinkNetException("未加载任何程序集。");

            foreach (var plug in _plugins) {
                plug.Initialize(_container, _assemblies);
            }

            CommandProcessorManager.Instance.Start();
            EventProcessorManager.Instance.Start();

            _assemblies.Clear();
            _plugins.Clear();
        }

        public Bootstrapper LoadAssemblies()
        {
            string applicationAssemblyDirectory = GetApplicationAssemblyDirectory();

            Directory.GetFiles(applicationAssemblyDirectory, "*.dll", SearchOption.TopDirectoryOnly)
                .Select(file => {
                    return Assembly.LoadFile(file);
                })
                .Where(assembly => {
                    return assembly.IsDefined(typeof(ParticipateInBootstrapAttribute), false) || AssemblyFilter(assembly);
                })
                .ToList()
                .ForEach(assembly => {
                    _assemblies.Add(assembly);
                });

            return this;
        }

        private static bool AssemblyFilter(Assembly assembly)
        {
            var assemblies = new string[] {
                    "ThinkNet",
                    "ThinkNet.Plugins.EntityFramework",
                    "ThinkNet.Plugins.NHibernate",
                    "ThinkNet.Plugins.Unity",
                    "Microsoft.Practices.ServiceLocation",
                    "Microsoft.Practices.Unity",
                    "Microsoft.Practices.Unity.Configuration",
                    "Microsoft.Practices.Unity.Interception",
                    "Microsoft.Practices.Unity.Interception.Configuration",
                    "Iesi.Collections",
                    "NHibernate",
                    "EntityFramework"
                };

            return !assemblies.Contains(assembly.GetName().Name);
        }

        private static string GetApplicationAssemblyDirectory()
        {
            string directory = AppDomain.CurrentDomain.BaseDirectory;

            if (HttpContext.Current == null) {
                return directory;
            }

            return Path.Combine(directory, "Bin");
        }

        private static bool IsComponent(Type type)
        {
            return type.IsClass && !type.IsAbstract && type.GetAttributes<ParticipateInContainerAttribute>(false).Any();// &&
            //!DefaultCommandExecutorProvider.IsCommandExecutor(type) && !DefaultEventHandlerProvider.IsEventHandler(type);
        }
    }
}
