﻿using System;
using System.Collections.Generic;
using System.Linq;
using ThinkNet.Event;
using ThinkNet.Domain;
using ThinkNet.Database;
using ThinkNet.Infrastructure;

namespace ThinkNet.Command
{
    /// <summary>
    /// 继承该类表示将使用EventSourcing
    /// </summary>
    /// <typeparam name="TCommand"></typeparam>
    public abstract class CommandHandler<TCommand> : DisposableObject, ICommandHandler<TCommand>
        where TCommand : class, ICommand
    {
        private readonly ICommandContext _commandContext;
        private readonly IEventStore _eventStore;
        //private readonly ICache _domainCache;
        private readonly IAggregateRootTypeProvider _aggregateRootTypeProvider;
        private readonly IEventSynchronizerProvider _eventSynchronizerProvider;
        private readonly IRepository _repository;
        private readonly IDatabase _database;
        private readonly IEventPublisher _eventPublisher;

        protected CommandHandler()
        {
            this._commandContext = ServiceLocator.GetInstance<ICommandContext>();
            this._eventStore = ServiceLocator.GetInstance<IEventStore>();
            //this._domainCache = ServiceLocator.GetInstance<ICache>();
            this._aggregateRootTypeProvider = ServiceLocator.GetInstance<IAggregateRootTypeProvider>();
            this._eventSynchronizerProvider = ServiceLocator.GetInstance<IEventSynchronizerProvider>();
            this._repository = ServiceLocator.GetInstance<IRepository>();
            this._eventPublisher = ServiceLocator.GetInstance<IEventPublisher>();
        }

        protected CommandHandler(IDatabase database)
            : this()
        {
            this._database = database;
        }


        protected override void Dispose(bool disposing)
        {
            if (disposing) {
                _commandContext.Clear();

                if (_database != null) _database.Dispose();
            }
        }

        /// <summary>
        /// 执行命令前的方法
        /// </summary>
        protected virtual void OnExecuting()
        { }

        /// <summary>
        /// 执行命令
        /// </summary>
        /// <param name="commandContext">当前命令的上下文</param>
        /// <param name="command"></param>
        protected abstract void Execute(ICommandContext commandContext, TCommand command);

        /// <summary>
        /// 执行命令后的方法
        /// </summary>
        protected virtual void OnExecuted()
        { }

        /// <summary>
        /// 当执行命令遇到异常的处理方法
        /// </summary>
        /// <param name="ex"></param>
        protected virtual void OnException(Exception ex)
        { }

        void ICommandHandler<TCommand>.Handle(TCommand command)
        {
            this.Execute(_commandContext, command);

            var trackedAggregateRoots = _commandContext.GetTrackedAggregateRoots();

            var events = trackedAggregateRoots.SelectMany(p => p.UncommittedEvents).ToArray();
            List<EventStream> eventStreams = new List<EventStream>();

            try {
                this.OnExecuting();

                foreach (var dirtyAggregateRoot in trackedAggregateRoots) {
                    var eventStream = CommitAggregate(dirtyAggregateRoot);
                    if (eventStream != null)
                        eventStreams.Add(eventStream);
                }

                this.OnExecuted();
            }
            catch (Exception ex) {
                this.OnException(ex);

                throw ex;
            }

            eventStreams.ForEach(eventStream => {
                RefreshMemoryCache(eventStream);
            });

            eventStreams.ForEach(eventStream => {
                _eventPublisher.Publish(eventStream);
            });
        }

        private EventStream CommitAggregate(ISourcedAggregateRoot aggregate)
        {
            if (!aggregate.UncommittedEvents.Any())
                return null;

            var eventStream = BuildEvents(aggregate);
            aggregate.ClearEvents();

            foreach (var @event in eventStream.Events) {
                _eventSynchronizerProvider.GetSynchronizers(@event.GetType()).ToList()
                    .ForEach(synchronizer => {
                        synchronizer.OnBeforePersisting(@event);
                    });
            }

            _eventStore.Append(eventStream);

            foreach (var @event in eventStream.Events) {
                _eventSynchronizerProvider.GetSynchronizers(@event.GetType()).ToList()
                    .ForEach(synchronizer => {
                        synchronizer.OnAfterPersisted(@event);
                    });
            }

            return eventStream;
        }

        private EventStream BuildEvents(ISourcedAggregateRoot aggregateRoot)
        {
            var uncommittedEvents = aggregateRoot.UncommittedEvents.ToList();
            var aggregateRootType = aggregateRoot.GetType();
            var aggregateRootName = _aggregateRootTypeProvider.GetAggregateRootTypeName(aggregateRootType);

            return new EventStream(
                aggregateRoot.UniqueId,
                aggregateRootName,
                aggregateRoot.Version + 1,
                DateTime.UtcNow,
                uncommittedEvents);
        }

        private void RefreshMemoryCache(EventStream eventStream)
        {
            var aggregateRootType = _aggregateRootTypeProvider.GetAggregateRootType(eventStream.AggregateRootName);

            if (aggregateRootType == null) {
                throw new Exception(string.Format("Could not find aggregate root type by aggregate root name {0}", eventStream.AggregateRootName));
            }

            ISourcedAggregateRoot aggregateRoot;

            if (eventStream.Version <= 1) {
                aggregateRoot = _aggregateRootTypeProvider.CreateAggregateRoot(aggregateRootType) as ISourcedAggregateRoot;
                aggregateRoot.ReplayEventStreams(new[] { eventStream });
                //_domainCache.Set(aggregateRoot);
                _repository.SaveOrUpdate(aggregateRoot);

                return;
            }

            //aggregateRoot = _domainCache.Get(eventStream.AggregateRootId, aggregateRootType);
            aggregateRoot = _repository.Get(aggregateRootType, eventStream.AggregateRootId);
            if (aggregateRoot == null) {
                //aggregateRoot = _repository.Get(aggregateRootType, eventStream.AggregateRootId);
                //if (aggregateRoot != null) {
                //    _domainCache.Set(aggregateRoot);
                //}
            }
            else if (aggregateRoot.Version + 1 == eventStream.Version) {
                aggregateRoot.ReplayEventStreams(new[] { eventStream });
                _repository.SaveOrUpdate(aggregateRoot);
            }
            else if (aggregateRoot.Version + 1 < eventStream.Version) {
                aggregateRoot = _repository.Get(aggregateRootType, eventStream.AggregateRootId);
                if (aggregateRoot != null) {
                    //_domainCache.Set(aggregateRoot);
                    _repository.SaveOrUpdate(aggregateRoot);
                }
            }
        }
    }
}
