﻿using System.Collections.Generic;
using System.Linq;

namespace ThinkNet.Command
{
    /// <summary>
    /// 命令处理程序管理器
    /// </summary>
    public sealed class CommandProcessorManager
    {
        private static readonly CommandProcessorManager _instance = new CommandProcessorManager();

        static CommandProcessorManager()
        { }

        /// <summary>
        /// 命令处理管理器的实例
        /// </summary>
        public static CommandProcessorManager Instance
        {
            get
            {
                return _instance;
            }
        }

        private readonly List<ICommandProcessor> _commandProcessors = new List<ICommandProcessor>();

        private CommandProcessorManager()
        { }

        /// <summary>
        /// 是否为一个空的处理器
        /// </summary>
        /// <returns></returns>
        public bool IsEmpty()
        {
            return !_commandProcessors.Any();
        }

        /// <summary>
        /// 获取所有可用的命令队列集合。
        /// </summary>
        public IEnumerable<ICommandQueue> GetCommandQueues()
        {
            return _commandProcessors.Select(x => x.BindingQueue);
        }

        /// <summary>
        /// 创建命令处理器
        /// </summary>
        /// <param name="commandProcessors"></param>
        public void Create(IEnumerable<ICommandProcessor> commandProcessors)
        {
            if (commandProcessors == null || !commandProcessors.Any())
                return;

            _commandProcessors.AddRange(commandProcessors);
        }

        /// <summary>
        /// 根据命令队列名称集合和线程数创建命令处理器
        /// </summary>
        /// <param name="commandQueueNames"></param>
        /// <param name="workCount"></param>
        public void CreateDefault(IEnumerable<string> commandQueueNames, int workCount)
        {
            this.Create(commandQueueNames.Select(queueName => {
                return new DefaultCommandProcessor(new DefaultCommandQueue(queueName), workCount);
            }).ToArray());
        }

        /// <summary>
        /// 启动命令处理器
        /// </summary>
        public void Start()
        {
            _commandProcessors.ForEach(processor => {
                processor.Initialize();
                processor.Start();
            });
        }
    }
}
