﻿using System;
using System.Threading;

using ThinkNet.Infrastructure;
using ThinkNet.Messaging;


namespace ThinkNet.Command
{
    /// <summary>
    /// 命令服务
    /// </summary>
    public sealed class CommandService
    {
        
        private static CommandService _instance = null;
        public static CommandService Instance
        {
            get
            {
                if (_instance != null)
                    return _instance;

                Interlocked.CompareExchange<CommandService>(ref _instance, 
                    new CommandService(ServiceLocator.GetInstance<ICommandQueueRouter>(), ServiceLocator.GetInstance<ICommandAsyncResultManager>()), null);

                return _instance;
            }
        }


        private readonly ICommandQueueRouter _commandQueueRouter;
        private readonly ICommandAsyncResultManager _commandAsyncResultManager;
        private CommandService(ICommandQueueRouter commandQueueRouter, ICommandAsyncResultManager commandAsyncResultManager)
        {
            _commandQueueRouter = commandQueueRouter;
            _commandAsyncResultManager = commandAsyncResultManager;
        }

        /// <summary>
        /// 发送命令(模拟同步)。
        /// </summary>
        /// <typeparam name="TCommand"></typeparam>
        /// <typeparam name="TResult"></typeparam>
        /// <param name="command"></param>
        /// <param name="func"></param>
        /// <returns></returns>
        public TResult Execute<TCommand, TResult>(TCommand command, Func<TCommand, TResult> func)
            where TCommand : class, ICommand, IMessage
        {
            var waitHandle = new ManualResetEvent(false);
            ErrorInfo errorInfo = null;

            this.Send(command, commandAsyncResult => {
                errorInfo = commandAsyncResult.ErrorInfo;

                waitHandle.Set();
            });

            waitHandle.WaitOne();

            if (errorInfo != null && errorInfo.ErrorMessage != null && errorInfo.Exception != null) {
                throw new CommandExecutionException(command.ID, typeof(TCommand), errorInfo.ErrorMessage, errorInfo.Exception);
            }
            if (errorInfo != null && errorInfo.ErrorMessage != null) {
                throw new CommandExecutionException(command.ID, typeof(TCommand), errorInfo.ErrorMessage);
            }
            if (errorInfo != null && errorInfo.Exception != null) {
                throw new CommandExecutionException(command.ID, typeof(TCommand), errorInfo.Exception);
            }

            return func(command);
        }


        /// <summary>
        /// 发送命令(模拟同步)。
        /// </summary>
        public void Execute<TCommand>(TCommand command)
            where TCommand : class, ICommand, IMessage
        {
            this.Execute(command, cmd => {
                return true;
            });
        }

        /// <summary>
        /// 发送命令(异步)
        /// </summary>
        /// <typeparam name="TCommand"></typeparam>
        /// <param name="command"></param>
        /// <param name="callback"></param>
        public void Send<TCommand>(TCommand command, Action<CommandAsyncResult> callback = null)
            where TCommand : class, ICommand, IMessage
        {
            command.MustNotNull("command");

            var commandQueue = _commandQueueRouter.Route(command);
            if (commandQueue == null) {
                throw new CommandQueueNotFoundException(command.GetType());
            }

            if (callback != null) {
                _commandAsyncResultManager.Add(command.ID, new CommandAsyncResult(callback));
            }
            commandQueue.Enqueue(command);
        }
    }
}
