﻿using System;
using System.Collections.Generic;
using System.Linq;

using ThinkNet.Domain;

namespace ThinkNet.Command
{
    public class DefaultCommandContext : ICommandContext
    {
        private readonly IList<ISourcedAggregateRoot> _trackingAggregateRoots = new List<ISourcedAggregateRoot>();
        private readonly IRepository _repository;

        /// <summary>Parameterized constructor.
        /// </summary>
        /// <param name="repository"></param>
        public DefaultCommandContext(IRepository repository)
        {
            _repository = repository;
        }

        /// <summary>Add an aggregate root to the context.
        /// </summary>
        /// <param name="aggregateRoot">The aggregate root to add.</param>
        /// <exception cref="ArgumentNullException">Throwed when the aggregate root is null.</exception>
        public void Add<TAggregateRoot>(TAggregateRoot aggregateRoot) where TAggregateRoot : class, ISourcedAggregateRoot
        {
            aggregateRoot.MustNotNull("aggregateRoot");

            _trackingAggregateRoots.Add(aggregateRoot);
        }
        /// <summary>Get the aggregate from the context.
        /// </summary>
        /// <param name="id">The id of the aggregate root.</param>
        /// <typeparam name="T">The type of the aggregate root.</typeparam>
        /// <returns>The found aggregate root.</returns>
        /// <exception cref="ArgumentNullException">Throwed when the id is null.</exception>
        /// <exception cref="AggregateRootNotFoundException">Throwed when the aggregate root not found.</exception>
        public TAggregateRoot Get<TAggregateRoot>(object id) where TAggregateRoot : class, ISourcedAggregateRoot
        {
            id.MustNotNull("id");
            
            var aggregateRoot = _trackingAggregateRoots.SingleOrDefault(x => x.UniqueId == id && x.GetType() == typeof(TAggregateRoot));
            if (aggregateRoot != null) {
                return aggregateRoot as TAggregateRoot;
            }

            aggregateRoot = _repository.Get<TAggregateRoot>(id);

            if (aggregateRoot != null) {
                _trackingAggregateRoots.Add(aggregateRoot);
            }

            return aggregateRoot as TAggregateRoot;
        }
        /// <summary>Returns all the tracked aggregate roots of the current context.
        /// </summary>
        /// <returns></returns>
        public IEnumerable<ISourcedAggregateRoot> GetTrackedAggregateRoots()
        {
            return _trackingAggregateRoots;
        }
        /// <summary>Clear all the tracked aggregate roots of the current context.
        /// </summary>
        public void Clear()
        {
            _trackingAggregateRoots.Clear();
        }
    }
}
