﻿using System;
using ThinkNet.Messaging;
using ThinkNet.Infrastructure;

namespace ThinkNet.Command
{
    public class DefaultCommandExecutor : MessageExecutor<ICommand, ICommandQueue>, ICommandExecutor
    {
        private readonly ICommandHandlerProvider _commandHandlerProvider;
        private readonly ICommandAsyncResultManager _commandAsyncResultManager;
        private readonly ILogger _logger;

        public DefaultCommandExecutor(ICommandHandlerProvider commandHandlerProvider, ICommandAsyncResultManager commandAsyncResultManager, ILogger logger)
        {
            this._commandHandlerProvider = commandHandlerProvider;
            this._commandAsyncResultManager = commandAsyncResultManager;
            this._logger = logger;
        }

        public override void Execute(ICommand command, ICommandQueue commandQueue)
        {
            var commandHandler = _commandHandlerProvider.GetCommandHandler(command.GetType());

            if (commandHandler == null) {
                var errorMessage = string.Format("Command handler not found for {0}", command.GetType().FullName);
                _logger.Fatal(errorMessage);
                _commandAsyncResultManager.TryComplete(command.ID, new ErrorInfo(errorMessage));
                FinishExecution(command, commandQueue);
                return;
            }

            try {
                commandHandler.Handle(command);

                _commandAsyncResultManager.TryComplete(command.ID, null);
            }
            catch (Exception ex) {
                var commandHandlerType = commandHandler.GetInnerCommandHandler().GetType();
                var errorMessage = string.Format("Exception raised when {0} handling {1}, command id:{2}.", commandHandlerType.Name, command.GetType().Name, command.ID);
                _logger.Error(errorMessage, ex);
                _commandAsyncResultManager.TryComplete(command.ID, new ErrorInfo(errorMessage, ex));
            }
            finally {
                if (commandHandler is IDisposable)
                    (commandHandler as IDisposable).Dispose();

                FinishExecution(command, commandQueue);
            }
        }
    }
}
