﻿using System;
using System.Collections;
#if NET40
using System.Collections.Concurrent;
#endif
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Linq;
using System.Text;

namespace ThinkNet.Database
{
    /// <summary>
    /// 执行数据后的动作
    /// </summary>
    public enum DbDataAction
    {
        /// <summary>
        /// 删除
        /// </summary>
        Delete,
        /// <summary>
        /// 新增保存
        /// </summary>
        Save,
        /// <summary>
        /// 更新
        /// </summary>
        Update,
    }

    /// <summary>
    /// 执行数据的事件类
    /// </summary>
    public class DbDataExecuteEventArgs : EventArgs
    {
        /// <summary>
        /// 构造函数
        /// </summary>
        /// <param name="db"></param>
        /// <param name="action"></param>
        public DbDataExecuteEventArgs(Database db, DbDataAction action)
        {
            Db = db;
            Action = action;
        }

        /// <summary>
        /// 执行数据的动作
        /// </summary>
        public DbDataAction Action { get; private set; }
        /// <summary>
        /// db
        /// </summary>
        public Database Db { get; private set; }
    }

    /// <summary>
    /// 执行数据的委托
    /// </summary>
    /// <param name="sender">发送对象</param>
    /// <param name="e"></param>
    public delegate void DbDataExecuteEventHandler(object sender, DbDataExecuteEventArgs e);


    /// <summary>
    /// 执行数据验证的事件类
    /// </summary>
    public class DbDataValidationEventArgs : DbDataExecuteEventArgs
    {
        /// <summary>
        /// 构造函数
        /// </summary>
        /// <param name="db"></param>
        /// <param name="action"></param>
        public DbDataValidationEventArgs(Database db, DbDataAction action)
            : base(db, action)
        { }

        /// <summary>
        /// 提交的数据
        /// </summary>
        public IDictionary<string, object> Data { get; internal set; }
    }

    /// <summary>
    /// 执行数据验证的委托
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public delegate bool DbDataValidationEventHandler(object sender, DbDataValidationEventArgs e);

    /// <summary>
    /// 数据访问基类
    /// </summary>
    public abstract class Database : DisposableObject, IDatabase
    {
        #region ado.net
        #region Private Static Methods
        static void CloseConnection(IDbConnection connection)
        {
            if (connection == null) {
                return;
            }

            if (connection.State != ConnectionState.Closed) {
                connection.Close();
            }
            connection.Dispose();
        }

        static void CloseTransaction(IDbTransaction transaction)
        {
            if (transaction == null) {
                return;
            }

            CloseConnection(transaction.Connection);

            transaction.Dispose();
        }

        /// <summary>
        /// 为当前对象创建一个数据连接
        /// </summary>
        /// <returns></returns>
        private static DbConnection CreateConnection(DbProviderFactory dbProviderFactory, string connectionString)
        {
            DbConnection connection = null;
            try {
                connection = dbProviderFactory.CreateConnection();
                connection.ConnectionString = connectionString;
                connection.Open();

                return connection;
            }
            catch (Exception ex) {
                CloseConnection(connection);
                throw ex;
            }
        }

        private static void AttachParameters(IDbCommand command, IDataParameter[] commandParameters)
        {
            command.MustNotNull("command");

            if (commandParameters != null) {
                foreach (IDataParameter p in commandParameters) {
                    if (p != null) {
                        if ((p.Direction == ParameterDirection.InputOutput || p.Direction == ParameterDirection.Input) &&
                            (p.Value == null)) {
                            p.Value = DBNull.Value;
                        }
                        command.Parameters.Add(p);
                    }
                }
            }
        }
        #endregion

        #region Private Methods
        protected IDbCommand CreateCommandByCommandType(CommandType commandType, string commandText, IDataParameter[] commandParameters)
        {
            IDbCommand command = _connection.CreateCommand();
            command.CommandType = commandType;
            command.CommandText = commandText;
            //command.Connection = _connection;
            if (_transaction != null) {
                command.Transaction = _transaction;
            }

            if (commandParameters != null) {
                AttachParameters(command, commandParameters);
            }

            return command;
        }

        /// <summary>
        /// 执行当前数据库连接对象的命令,指定参数.
        /// </summary>
        /// <remarks>
        /// 示例: int result = ExecuteNonQuery(out id, CommandType.StoredProcedure, "PublishOrders", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="id">自动增量值</param>
        /// <param name="commandType">命令类型 (存储过程,命令文本,其它.)</param>
        /// <param name="commandText">存储过程名称或SQL语句</param>
        /// <param name="commandParameters">分配给命令的DbParamter参数数组(无参数请写(DbParameter[])null)</param>
        /// <returns>返回命令影响的行数</returns>
        private int ExecuteNonQuery(out int id, CommandType commandType, string commandText, params IDataParameter[] commandParameters)
        {
            commandText.MustNotEmpty("commandText");

            using (IDbCommand command = CreateCommandByCommandType(commandType, commandText, commandParameters)) {

                try {
                    int rowsAffected = command.ExecuteNonQuery();

                    command.Parameters.Clear();

                    command.CommandText = this.GetIdentitySql();
                    command.CommandType = CommandType.Text;

                    id = Convert.ToInt32(command.ExecuteScalar() ?? 0);

                    return rowsAffected;
                }
                catch (Exception ex) {
                    throw ex;
                }
            }
        }
        #endregion

        #region Members
        //private readonly DbProviderFactory _dbProviderFactory;
        private IDbConnection _connection;
        public IDbConnection DbConnection
        {
            get { return this._connection; }
        }
        private IDbTransaction _transaction;
        public IDbTransaction DbTransaction
        {
            get { return this._transaction; }
        }
        #endregion

        #region Ctor
        /// <summary>
        /// 创建 <see cref="Database"/> 实例
        /// </summary>
        /// <param name="dbProviderFactory"></param>
        protected Database(DbProviderFactory dbProviderFactory, string connectionString)
        {
            connectionString.MustNotEmpty("connectionString");

            //_dbProviderFactory = dbProviderFactory;
            _connection = CreateConnection(dbProviderFactory, connectionString);
        }

        protected Database(IDbConnection connection)
        {
            connection.MustNotNull("connection");

            _connection = connection;
        }
        #endregion


        #region Methods
        /// <summary>
        /// 检索SQL参数信息并填充
        /// </summary>
        /// <param name="discoveryCommand"></param>
        protected abstract void DeriveParameters(DbCommand discoveryCommand);

        /// <summary>
        /// 自动增量T-SQL
        /// </summary>
        protected abstract string GetIdentitySql();

        /// <summary>
        /// 获取表字典信息
        /// </summary>
        /// <param name="tableName">表名</param>
        /// <returns>模型字段集合</returns>
        protected abstract DbColumn[] GetTableColumns(string tableName);


        //public Database Connection(IDbConnection connection)
        //{
        //    CloseTransaction(_transaction);
        //    CloseConnection(_connection);

        //    _connection = connection;
        //    if (_connection.State != ConnectionState.Open) {
        //        _connection.Open();
        //    }

        //    return this;
        //}

        /// <summary>
        /// 开始事务
        /// </summary>
        public IDbTransaction BeginTransaction()
        {
            if (_transaction == null) {
                _transaction = _connection.BeginTransaction();
            }

            return _transaction;
        }

        /// <summary>
        /// 执行当前数据库连接对象的命令,指定参数.
        /// </summary>
        /// <remarks>
        /// 示例: int result = ExecuteNonQuery(CommandType.StoredProcedure, "PublishOrders", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="commandType">命令类型 (存储过程,命令文本, 其它.)</param>
        /// <param name="commandText">存储过程名称或SQL语句</param>
        /// <param name="commandParameters">分配给命令的DbParamter参数数组(无参数请写(DbParameter[])null)</param>
        /// <returns>返回命令影响的行数</returns>
        public int ExecuteNonQuery(CommandType commandType, string commandText, params IDataParameter[] commandParameters)
        {
            commandText.MustNotEmpty("commandText");


            using (IDbCommand command = CreateCommandByCommandType(commandType, commandText, commandParameters)) {

                try {
                    return command.ExecuteNonQuery();
                }
                catch (Exception ex) {
                    throw ex;
                }
                finally {
                    command.Parameters.Clear();
                }
            }
        }
        /// <summary>
        /// 执行当前数据库连接对象的数据阅读器,指定参数.
        /// </summary>
        /// <remarks>
        /// 示例: IDataReader dr = ExecuteReader(CommandType.StoredProcedure, "GetOrders", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="commandType">命令类型 (存储过程,命令文本或其它)</param>
        /// <param name="commandText">存储过程名或SQL语句</param>
        /// <param name="commandParameters">分配给命令的DbParamter参数数组(无参数请写(DbParameter[])null)</param>
        /// <returns>返回包含结果集的IDataReader</returns>
        public IDataReader ExecuteReader(CommandType commandType, string commandText, params IDataParameter[] commandParameters)
        {
            commandText.MustNotEmpty("commandText");


            using (IDbCommand command = CreateCommandByCommandType(commandType, commandText, commandParameters)) {
                try {
                    return command.ExecuteReader();
                }
                catch (Exception ex) {
                    throw ex;
                }
                finally {
                    bool canClear = true;
                    foreach (DbParameter commandParameter in command.Parameters) {
                        if (commandParameter.Direction != ParameterDirection.Input)
                            canClear = false;
                    }

                    if (canClear) {
                        command.Parameters.Clear();
                    }
                }
            }
        }
        /// <summary>
        /// 执行指定数据库连接对象的命令,指定参数,返回结果集中的第一行第一列.
        /// </summary>
        /// <remarks>
        /// 示例: int orderCount = (int)ExecuteScalar(CommandType.StoredProcedure, "GetOrderCount", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="commandType">命令类型 (存储过程,命令文本或其它)</param>
        /// <param name="commandText">存储过程名称或SQL语句</param>
        /// <param name="commandParameters">分配给命令的DbParamter参数数组(无参数请写(DbParameter[])null)</param>
        /// <returns>返回结果集中的第一行第一列</returns>
        public object ExecuteScalar(CommandType commandType, string commandText, params IDataParameter[] commandParameters)
        {
            commandText.MustNotEmpty("commandText");


            using (IDbCommand command = CreateCommandByCommandType(commandType, commandText, commandParameters)) {
                try {
                    return command.ExecuteScalar();
                }
                catch (Exception ex) {
                    throw ex;
                }
                finally {
                    command.Parameters.Clear();
                }
            }
        }
        /// <summary>
        /// 执行指定数据库连接字符串的命令,指定参数值.返回DataSet.
        /// </summary>
        /// <remarks>
        /// 示例: DataSet ds = ExecuteDataset(CommandType.StoredProcedure, "GetOrders", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="commandType">命令类型 (存储过程,命令文本或其它)</param>
        /// <param name="commandText">存储过程名称或SQL语句</param>
        /// <param name="commandParameters">分配给命令的DbParamter参数数组(无参数请写(DbParameter[])null)</param>
        /// <returns>返回一个包含结果集的DataSet</returns>
        public abstract DataSet ExecuteDataset(CommandType commandType, string commandText, params IDataParameter[] commandParameters);

        /// <summary>
        /// Command对象的传入参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">数据类型</param>
        /// <param name="size">数据长度</param>
        /// <param name="value">值</param>
        /// <returns></returns>
        public IDataParameter CreateInParameter(string paramName, DbType dbType, int size, object value)
        {
            return CreateParameter(paramName, dbType, ParameterDirection.Input, size, value);
        }
        /// <summary>
        /// Command对象的传入参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">数据类型</param>
        /// <param name="value">值</param>
        /// <returns></returns>
        public IDataParameter CreateInParameter(string paramName, DbType dbType, object value)
        {
            return CreateInParameter(paramName, dbType, -1, value);
        }
        /// <summary>
        /// Command对象的传出参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">数据类型</param>
        /// <param name="size">数据长度</param>
        /// <returns></returns>
        public IDataParameter CreateOutParameter(string paramName, DbType dbType, int size)
        {
            return CreateParameter(paramName, dbType, ParameterDirection.Output, size, null);
        }
        /// <summary>
        /// Command对象的传出参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">数据类型</param>
        /// <returns></returns>
        public IDataParameter CreateOutParameter(string paramName, DbType dbType)
        {
            return CreateOutParameter(paramName, dbType, -1);
        }

        /// <summary>
        /// 创建Commond用的DbParameter参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">参数对应的数据类型枚举</param>
        /// <param name="direction">参数类型</param>
        /// <param name="size">大小</param>
        /// <param name="value">值</param>
        /// <returns></returns>
        protected abstract DbParameter CreateParameter(string paramName, DbType dbType, ParameterDirection direction, int size, object value);

        /// <summary>
        /// Builds a value parameter name for the current database.
        /// </summary>
        /// <param name="name">The name of the parameter.</param>
        /// <returns>A correctly formated parameter name.</returns>
        protected virtual string BuildParameterName(string name)
        {
            return name;
        }
        #endregion

        #region IDisposable 成员
        protected override void Dispose(bool disposing)
        {
            this.Reset();

            if (disposing) {
                CloseConnection(_connection);
                CloseTransaction(_transaction);
            }
        }

        #endregion

#if NET40
        static ConcurrentDictionary<string, DbColumn[]> _dbColumnsCache = new ConcurrentDictionary<string, DbColumn[]>();
#endif
#if NET35
        static Hashtable _dbColumnsCache = Hashtable.Synchronized(new Hashtable());
#endif
        /// <summary>
        /// 获取当前表的列结构(有缓存)
        /// </summary>
        /// <returns></returns>
        private DbColumn[] GetTableColumnsWithCache()
        {
            if (string.IsNullOrEmpty(_tableName))
                throw new DatabaseException(Resources.EX_DATABASE_NOTSET_TABLENAME);

            string key = string.Format("Db:{0}:{1}", _connection.ConnectionString.GetHashCode(), _tableName);

#if NET40
            DbColumn[] columns;

            if (!_dbColumnsCache.TryGetValue(key, out columns)) {
                columns = this.GetTableColumns(_tableName);

                _dbColumnsCache.TryAdd(key, columns);
            }

            return columns;
#endif

#if NET35
            DbColumn[] columns = _dbColumnsCache[hashKey] as DbColumn[];
            if (columns == null) {
                columns = this.GetTableColumns(tableName);

                _dbColumnsCache[hashKey] = columns;
            }

            return columns;
#endif
        }

        private string GetPrimaryKey()
        {
            DbColumn[] pk = this.GetTableColumnsWithCache().Where(p => p.Primary).ToArray();

            if (pk.Length == 0)
                throw new DatabaseException(Resources.EX_DATABASE_NOTDEFINE_TABLE_PRIMARYKEY, _tableName);
            if (pk.Length > 1)
                throw new DatabaseException(Resources.EX_DATABASE_TABLE_HAVE_MOREPK, _tableName);

            StringBuilder sb = new StringBuilder();
            if (_joins.Length > 0) {
                sb.Append(_tableName).Append(".");
            }
            sb.Append(pk[0].ColumnName);

            return sb.ToString();
        }
        #endregion

        #region sql

        private SqlJoin[] _joins = new SqlJoin[0];
        /// <summary>
        /// 连接多表
        /// </summary>
        /// <param name="joins"><see cref="SqlJoin"/>类的数组</param>
        /// <returns>当前对象</returns>
        public Database Join(params SqlJoin[] joins)
        {
            if (joins != null && joins.Length > 0) {
                _joins = joins;
            }

            return this;
        }

        private string _tableName; //当启用Table方法时
        /// <summary>
        /// 设置表名称
        /// </summary>
        /// <param name="tableName">源表名称</param>
        public Database Table(string tableName)
        {
            _tableName = tableName;

            return this;
        }

        private bool _distinct = false;
        /// <summary>
        /// 过滤重复记录
        /// </summary>
        /// <returns></returns>
        public Database Distinct()
        {
            _distinct = true;
            return this;
        }

        private SqlField[] _fields = new SqlField[0];
        /// <summary>
        /// 设置显示字段
        /// </summary>
        /// <param name="fields"><see cref="SqlField"/>类的数组</param>
        public Database Field(params SqlField[] fields)
        {
            if (fields != null && fields.Length > 0) {
                _fields = fields;
            }
            return this;
        }

        private SqlWhere[] _filters = new SqlWhere[0];
        /// <summary>
        /// 设置查询条件
        /// </summary>
        /// <param name="condition">查询类数组</param>
        public Database Where(params SqlWhere[] conditions)
        {
            if (conditions != null && conditions.Length > 0) {
                _filters = conditions;
            }

            return this;
        }

        private SqlGroup[] _groups = new SqlGroup[0];
        /// <summary>
        /// 设定分组字段来自Fields
        /// </summary>
        public Database GroupBy()
        {
            _groups = Array.FindAll(_fields, p => p.Aggrefunc == DbAggrefunc.NONE);

            return this;
        }

        private SqlHaving[] _havings = new SqlHaving[0];
        /// <summary>
        /// 设置合计函数过滤条件
        /// </summary>
        /// <param name="conditions">过滤条件数组</param>
        /// <returns></returns>
        public Database Having(params SqlHaving[] conditions)
        {
            if (conditions != null && conditions.Length > 0) {
                _havings = conditions;
            }
            return this;
        }

        private SqlOrder[] _sorters = new SqlOrder[0];
        /// <summary>
        /// 排序
        /// </summary>
        /// <param name="orders">排序类数组</param>
        public Database OrderBy(params SqlOrder[] orders)
        {
            if (orders != null && orders.Length > 0) {
                _sorters = orders;
            }
            return this;
        }

        private int _pageIndex;
        private int _pageSize;
        /// <summary>
        /// 显示分页数据
        /// </summary>
        /// <param name="page">页码(从1开始)</param>
        /// <param name="rows"></param>
        /// <returns></returns>
        public Database Limit(int page, int rows)
        {
            _pageIndex = page - 1;
            if (_pageIndex < 0) {
                _pageIndex = 0;
            }
            _pageSize = rows;
            return this;
        }

        protected abstract string PagingSupport(int pageIndex, int pageSize);

        //private string PagingSupport(int pageIndex, int pageSize)
        //{
        //    if (_dbType == DbServerType.MySql) {
        //        return string.Format("SELECT%DISTINCT%%FIELDS%%TABLE%%JOIN%%WHERE%%GROUP%%HAVING%%ORDER% LIMIT {0},{1}", pageIndex * pageSize, pageSize);
        //    }
        //    else if (_dbType == DbServerType.SQLite) {
        //        return string.Format("SELECT%DISTINCT%%FIELD%%TABLE%%JOIN%%WHERE%%GROUP%%HAVING%%ORDER% LIMIT {0} OFFSET {1}", pageSize, pageIndex * pageSize);
        //    }
        //    else {
        //        if (pageIndex > 0) {
        //            return string.Format("SELECT * FROM (SELECT ROW_NUMBER() OVER (ORDER BY Sys_RowIndex) AS Sys_RowNum,temp1.* FROM ({0}) AS temp1) AS temp2 WHERE Sys_RowNum BETWEEN {1} AND {2}",
        //                "SELECT%DISTINCT% TOP (100) PERCENT%FIELDS%, ROW_NUMBER() OVER (%ORDER%) AS Sys_RowIndex%TABLE%%JOIN%%WHERE%%GROUP%%HAVING%",
        //                pageIndex * pageSize + 1, pageIndex * pageSize);
        //        }
        //        else {
        //            return string.Format("SELECT TOP {0} * FROM (SELECT%DISTINCT% TOP (100) PERCENT%FIELDS%%TABLE%%JOIN%%WHERE%%GROUP%%HAVING%%ORDER%) AS temp1", pageSize);
        //        }
        //    }
        //}

        private string GetSelectSql(StringBuilder tsql)
        {
            if (_pageIndex > 0 || _pageSize > 0) {
                tsql.Append(this.PagingSupport(_pageIndex, _pageSize));
            }
            else {
                tsql.Append("SELECT%DISTINCT%%FIELDS%%TABLE%%JOIN%%WHERE%%GROUP%%HAVING%%ORDER%");
            }

            string tsql_table = string.Concat(" FROM ", _tableName);
            //if (!string.IsNullOrEmpty(_tableAlias) && _joins.Length > 0) {
            //    tsql_table += string.Format(" AS {0}", _tableAlias);
            //}



            tsql.Replace("%DISTINCT%", _distinct ? " DISTINCT" : string.Empty);
            tsql.Replace("%FIELDS%", Restrictions.OutputSqlString(_fields));
            tsql.Replace("%TABLE%", tsql_table);
            tsql.Replace("%JOIN%", this.OutputSqlString(_joins));
            tsql.Replace("%WHERE%", this.OutputSqlString(_filters, " WHERE "));
            tsql.Replace("%GROUP%", Restrictions.OutputSqlString(_groups));
            tsql.Replace("%HAVING%", this.OutputSqlString(_havings, " HAVING "));
            tsql.Replace("%ORDER%", Restrictions.OutputSqlString(_sorters));

            return tsql.ToString();
        }

        private DataSourceType ExecuteResult<DataSourceType>(CommandType commandType, string commandText, params IDataParameter[] commandParameters)
        {
            object obj = null;
            if (typeof(DataSourceType) == typeof(DataSet)) {
                obj = this.ExecuteDataset(commandType, commandText, commandParameters);
            }
            else if (typeof(DataSourceType) == typeof(DataTable)) {
                DataSet ds = this.ExecuteDataset(commandType, commandText, commandParameters);
                if (ds != null && ds.Tables.Count > 0) {
                    obj = ds.Tables[0];
                }
            }
            else if (typeof(IDataReader).IsAssignableFrom(typeof(DataSourceType))) {
                obj = this.ExecuteReader(commandType, commandText, commandParameters);
            }
            else if (typeof(DataSourceType) == typeof(object)) {
                obj = this.ExecuteScalar(commandType, commandText, commandParameters);
            }
            else if (typeof(DataSourceType) == typeof(int)) {
                obj = this.ExecuteNonQuery(commandType, commandText, commandParameters);
            }
            else {
                throw new DatabaseException(Resources.EX_DATABASE_NOTSUPPORT_DATATYPE);
            }

            this.Reset();

            return (DataSourceType)obj;
        }

        protected virtual void Reset()
        {
            _joins = new SqlJoin[0];
            _havings = new SqlHaving[0];
            _groups = new SqlGroup[0];
            _sorters = new SqlOrder[0];
            _fields = new SqlField[0];
            _filters = new SqlWhere[0];

            _pageIndex = 0;
            _pageSize = 0;

            _distinct = false;
            _tableName = null;            

            _parameters.Clear();
        }

        private DataSourceType Select<DataSourceType>()
        {
            string sql = this.GetSelectSql(new StringBuilder());


            return this.ExecuteResult<DataSourceType>(CommandType.Text, sql, _parameters.ToArray());
        }

        protected abstract string BuildLikeParameterName(string parameterName);

        List<IDataParameter> _parameters = new List<IDataParameter>();
        private string OutputSql(SqlWhere expression)
        {
            if (expression == null || expression.Value == null || expression.Value.ToString().Trim().Length == 0)
                return string.Empty;

            StringBuilder sb = new StringBuilder(expression.FieldName);

            if (expression.Aggrefunc != DbAggrefunc.NONE) {
                sb.Insert(0, "(").Insert(0, expression.Aggrefunc).Append(")");
            }

            sb.Append(Restrictions.symbol[expression.Symbol]);

            if (expression.Safety) {
                return sb.Append(expression.Value).ToString();
            }

            DbType dbtype = Array.Find(this.GetTableColumns(expression.Prefix ?? _tableName), p => p.ColumnName == expression.Name).GetDbTypeForParameter();

            if (expression.Symbol == DbJudgeSymbol.IN || expression.Symbol == DbJudgeSymbol.NIN) {
                sb.Append("(");
                Array.ForEach(expression.Value as object[], obj => {
                    IDataParameter parame = this.CreateInParameter(string.Format("p{0}", _parameters.Count), dbtype, obj);
                    _parameters.Add(parame);

                    sb.Append(parame.ParameterName).Append(",");
                });
                sb.Remove(sb.Length - 1, 1).Append(")");
            }
            else if (expression.Symbol == DbJudgeSymbol.LIKE || expression.Symbol == DbJudgeSymbol.NLIKE) {
                IDataParameter parame = this.CreateInParameter(string.Format("p{0}", _parameters.Count), dbtype, expression.Value);
                _parameters.Add(parame);

                sb.Append(this.BuildLikeParameterName(parame.ParameterName));
                //switch (DbServer) {
                //    case DbServerType.SqlServer:
                //        sb.Append("('%'+").Append(parame.ParameterName).Append("+'%')");
                //        break;
                //    case DbServerType.MySql:
                //        sb.Append("concat('%',").Append(parame.ParameterName).Append(",'%')");
                //        break;
                //}
            }
            else {
                IDataParameter parame = this.CreateInParameter(string.Format("p{0}", _parameters.Count), dbtype, expression.Value);
                _parameters.Add(parame);

                sb.Append(parame.ParameterName);
            }



            return sb.ToString();
        }

        private string OutputSqlString(SqlWhere[] conditions, string keyword)
        {
            int total = conditions.Length;
            if (total == 0)
                return string.Empty;

            StringBuilder sb = new StringBuilder();
            sb.Append("(");
            for (int i = 0; i < total; i++) {
                SqlWhere criteria = conditions[i];

                string result = OutputSql(criteria);

                if (string.IsNullOrEmpty(result)) {
                    continue;
                }

                if (i > 0 && sb.Length > 1 && i < total) {
                    sb.Append(criteria.Logical == DbLogical.And ? ") AND (" : " OR ");
                }

                sb.Append(result);
            }
            sb.Append(")");

            if (sb.Length == 2)
                return string.Empty;


            return sb.Insert(0, keyword).ToString();
        }

        private string OutputSqlString(SqlJoin[] joins)
        {
            StringBuilder sb = new StringBuilder();

            foreach (SqlJoin item in joins) {
                sb.AppendFormat(" {0} JOIN {1}", item.JoinType, item.Table)
                    .Append(OutputSqlString(item.OnConditions, " ON "));
            }

            return sb.ToString();
        }

        #endregion

        #region curd
        /// <summary>
        /// 查询结果
        /// </summary>
        /// <returns></returns>
        public IDictionary Find()
        {
            using (IDataReader reader = this.Select<IDataReader>()) {
                if (reader.Read()) {
                    return reader.ToDictionary();
                }
            }

            return null;
        }
        /// <summary>
        /// 查询结果
        /// </summary>
        /// <param name="id">主键id</param>
        /// <returns></returns>
        public IDictionary Find(object id)
        {
            return this.Where(new SqlWhere(this.GetPrimaryKey(), id)).Find();
        }
        /// <summary>
        /// 查询结果集
        /// </summary>
        /// <returns></returns>
        public IList FindAll()
        {
            using (IDataReader reader = this.Select<IDataReader>()) {
                return reader.ToList();
            }
        }
        /// <summary>
        /// 查询结果集
        /// </summary>
        /// <param name="ids">主键值数组</param>
        /// <returns></returns>
        public IList FindAll(params object[] ids)
        {
            return this.Where(new SqlWhere(this.GetPrimaryKey(), DbJudgeSymbol.IN, ids)).FindAll();
        }
        /// <summary>
        /// 查询结果
        /// </summary>
        /// <param name="total">记录总数</param>
        /// <returns></returns>
        public IList FindAll(out int total)
        {
            string sql = this.GetSelectSql(
                new StringBuilder().Append("SELECT COUNT(0) FROM (SELECT%DISTINCT%%FIELDS%%TABLE%%JOIN%%WHERE%%GROUP%%HAVING%) app_total;")
            );

            using (IDataReader reader = this.Select<IDataReader>()) {
                if (reader.NextResult() && reader.Read()) {
                    total = Convert.ToInt32(reader.GetValue(0));
                }
                else {
                    total = 0;
                }

                if (reader.NextResult()) {
                    return reader.ToList();
                }
                else {
                    return null;
                }
            }
        }


        public event DbDataValidationEventHandler DataValidated;
        protected virtual bool OnDataValidated(DbDataValidationEventArgs e)
        {
            if (DataValidated != null) {
                return DataValidated(this, e);
            }

            return true;
        }


        /// <summary>
        /// 在数据执行操作后发生
        /// </summary>
        public event DbDataExecuteEventHandler DataExecuted;
        /// <summary>
        /// 引发 <see cref="DataExecuted"/> 事件
        /// </summary>
        /// <param name="e"></param>
        protected virtual void OnDataExecuted(DbDataExecuteEventArgs e)
        {
            if (DataExecuted != null) {
                DataExecuted(this, e);
            }
        }

        /// <summary>
        /// 在数据执行操作前发生
        /// </summary>
        public event DbDataExecuteEventHandler DataExecuting;
        /// <summary>
        /// 引发 <see cref="DataExecuting"/> 事件
        /// </summary>
        /// <param name="e"></param>
        protected virtual void OnDataExecuting(DbDataExecuteEventArgs e)
        {
            if (DataExecuting != null) {
                DataExecuting(this, e);
            }
        }

        private SqlData[] _datas = new SqlData[0];
        ///// <summary>
        ///// 设置添加/修改数据
        ///// </summary>
        ///// <param name="data">字段与值的集合</param>
        //public IDatabase Data(IDictionary data)
        //{
        //    if (data == null || data.Count == 0)
        //        return this;

        //    SqlData[] _data = new SqlData[data.Count];
        //    int index = 0;
        //    foreach (KeyValuePair<string, object> item in data) {
        //        _data[index++] = new SqlData(item.Key, item.Value);
        //    }

        //    return this.Data(_data);
        //}

        /// <summary>
        /// 设置添加/修改数据
        /// </summary>
        /// <param name="data"><see cref="TsqlData"/>类的数组形式</param>
        /// <returns></returns>
        public Database Data(params SqlData[] datas)
        {
            if (datas != null && datas.Length > 0) {
                _datas = datas;
            }

            return this;
        }

        private DbDataValidationEventArgs CreateEventArgs(DbDataAction action)
        {
            DbDataValidationEventArgs e = new DbDataValidationEventArgs(this, action);
            e.Data = new Dictionary<string, object>(_datas.Length);
            Array.ForEach(_datas, delegate(SqlData data) {
                e.Data.Add(data.Name, data.Value);
            });

            return e;
        }


        /// <summary>
        /// 保存数据
        /// </summary>
        /// <returns></returns>
        public bool Save()
        {
            if (_datas.Length <= 0)
                throw new DatabaseException(Resources.EX_DATABASE_DATAITEM_CHECK);

            DbDataValidationEventArgs e = this.CreateEventArgs(DbDataAction.Save);

            bool validate = this.OnDataValidated(e);

            if (!validate) {
                return false;
            }


            List<string> fields = new List<string>();
            List<string> values = new List<string>();

            foreach (SqlData data in _datas) {
                var column = this.GetTableColumnsWithCache().FirstOrDefault(p => p.ColumnName == data.Name);

                if (column == null)
                    continue;

                if (column.AutoIncrement)
                    continue;

                fields.Add(data.Name);

                if (data.Safety) {
                    values.Add(data.Value.ToString());
                    continue;
                }

                IDataParameter parame = this.CreateInParameter(string.Format("p{0}", _parameters.Count), column.GetDbTypeForParameter(), data.Value);
                _parameters.Add(parame);

                values.Add(parame.ParameterName);
            }

            string strsql = string.Format("INSERT INTO {0}({1}) VALUES({2})", _tableName,
                string.Join(",", fields.ToArray()), string.Join(",", values.ToArray()));
            this.Reset();

            this.OnDataExecuting(e);
            int rowsAffected = this.ExecuteNonQuery(CommandType.Text, strsql, _parameters.ToArray());
            this.OnDataExecuted(e);

            return rowsAffected.Equals(0);
        }

        /// <summary>
        /// 保存数据并输出自动增量
        /// </summary>
        /// <param name="id">自动增量</param>
        /// <returns></returns>
        public bool Save(out int id)
        {
            if (_datas.Length <= 0)
                throw new DatabaseException(Resources.EX_DATABASE_DATAITEM_CHECK);

            DbDataValidationEventArgs e = this.CreateEventArgs(DbDataAction.Save);

            bool validate = this.OnDataValidated(e);

            if (!validate) {
                id = 0;
                return false;
            }


            List<string> fields = new List<string>();
            List<string> values = new List<string>();

            foreach (SqlData data in _datas) {
                var column = this.GetTableColumnsWithCache().FirstOrDefault(p => p.ColumnName == data.Name);

                if (column == null)
                    continue;

                if (column.AutoIncrement)
                    continue;

                fields.Add(data.Name);

                if (data.Safety) {
                    values.Add(data.Value.ToString());
                    continue;
                }

                IDataParameter parame = this.CreateInParameter(string.Format("p{0}", _parameters.Count), column.GetDbTypeForParameter(), data.Value);
                _parameters.Add(parame);

                values.Add(parame.ParameterName);
            }

            string strsql = string.Format("INSERT INTO {0}({1}) VALUES({2})", _tableName,
                string.Join(",", fields.ToArray()), string.Join(",", values.ToArray()));
            this.Reset();

            this.OnDataExecuting(e);
            int rowsAffected = this.ExecuteNonQuery(out id, CommandType.Text, strsql, _parameters.ToArray());
            this.OnDataExecuted(e);

            return rowsAffected.Equals(0);
        }

        /// <summary>
        /// 更新数据
        /// </summary>
        /// <returns></returns>
        public bool Update()
        {
            if (_datas.Length <= 0)
                throw new ArgumentException(Resources.EX_DATABASE_DATAITEM_CHECK);

            DbDataValidationEventArgs e = this.CreateEventArgs(DbDataAction.Update);

            bool validate = this.OnDataValidated(e);

            if (!validate) {
                return false;
            }


            List<string> fields = new List<string>();
            List<SqlWhere> conditions = new List<SqlWhere>();

            foreach (SqlData data in _datas) {
                var column = this.GetTableColumnsWithCache().FirstOrDefault(p => p.ColumnName == data.Name);

                if (column == null)
                    continue;

                if (column.Primary || column.AutoIncrement) {
                    conditions.Add(new SqlWhere(data.Name, data.Value));
                    continue;
                }

                if (data.Safety) {
                    fields.Add(string.Format("{0}={1}", data.Name, data.Value));
                    continue;
                }


                IDataParameter parame = this.CreateInParameter(string.Format("p{0}", _parameters.Count), column.GetDbTypeForParameter(), data.Value);
                _parameters.Add(parame);

                fields.Add(string.Format("{0}={1}", data.Name, parame.ParameterName));
            }

            if (conditions.Any()) {
                _filters = conditions.ToArray();
            }

            if (_filters.Length <= 0)
                throw new DatabaseException(Resources.EX_DATABASE_NOTCONDITION);


            string strsql = string.Format("UPDATE {0} SET {1} WHERE {2}", _tableName,
                string.Join(",", fields.ToArray()), this.OutputSqlString(_filters, null));
            this.Reset();

            this.OnDataExecuting(e);
            int rowsAffected = this.ExecuteNonQuery(CommandType.Text, strsql, _parameters.ToArray());
            this.OnDataExecuted(e);

            return rowsAffected.Equals(0);
        }

        /// <summary>
        /// 删除操作
        /// </summary>
        public bool Delete()
        {
            if (_filters.Length <= 0)
                throw new ArgumentException(Resources.EX_DATABASE_NOTCONDITION);

            string strsql = string.Format("DELETE FROM {0} WHERE {1}", _tableName, this.OutputSqlString(_filters, null));
            this.Reset();


            DbDataExecuteEventArgs e = new DbDataExecuteEventArgs(this, DbDataAction.Delete);

            this.OnDataExecuting(e);
            int rowsAffected = this.ExecuteNonQuery(CommandType.Text, strsql, _parameters.ToArray());
            this.OnDataExecuted(e);

            return rowsAffected.Equals(0);
        }

        /// <summary>
        /// 根据主键ID删除数据
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        public bool Delete(params object[] ids)
        {
            return this.Where(new SqlWhere(this.GetPrimaryKey(), DbJudgeSymbol.IN, ids)).Delete();
        }
        #endregion

        #region IDatabase 成员

        IDatabase IDatabase.Table(string table)
        {
            return this.Table(table);
        }

        IDatabase IDatabase.Data(params SqlData[] datas)
        {
            return this.Data(datas);
        }

        IDatabase IDatabase.Distinct()
        {
            return this.Distinct();
        }

        IDatabase IDatabase.Field(params SqlField[] fields)
        {
            return this.Field(fields);
        }

        IDatabase IDatabase.Join(params SqlJoin[] joins)
        {
            return this.Join(joins);
        }

        IDatabase IDatabase.Where(params SqlWhere[] conditions)
        {
            return this.Where(conditions);
        }

        IDatabase IDatabase.GroupBy()
        {
            return this.GroupBy();
        }

        IDatabase IDatabase.Having(params SqlHaving[] conditions)
        {
            return this.Having(conditions);
        }

        IDatabase IDatabase.OrderBy(params SqlOrder[] orders)
        {
            return this.OrderBy(orders);
        }

        IDatabase IDatabase.Limit(int page, int rows)
        {
            return this.Limit(page, rows);
        }
        #endregion
    }
}
