﻿using System;
using System.Collections;
using System.Data;

namespace ThinkNet.Database
{
    /// <summary>
    /// Ado.Net
    /// </summary>
    public interface IDatabase : IDisposable
    {
        /// <summary>
        /// 执行当前数据库连接对象的命令,指定参数.
        /// </summary>
        /// <remarks>
        /// 示例: int result = ExecuteNonQuery(CommandType.StoredProcedure, "PublishOrders", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="commandType">命令类型 (存储过程,命令文本,其它.)</param>
        /// <param name="commandText">存储过程名称或SQL语句</param>
        /// <param name="commandParameters">分配给命令的DbParamter参数数组(无参数请写(DbParameter[])null)</param>
        /// <returns>返回命令影响的行数</returns>
        int ExecuteNonQuery(CommandType commandType, string commandText, params IDataParameter[] commandParameters);

        /// <summary>
        /// 执行当前数据库连接对象的数据阅读器,指定参数.
        /// </summary>
        /// <remarks>
        /// 示例: IDataReader dr = ExecuteReader(CommandType.StoredProcedure, "GetOrders", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="commandType">命令类型 (存储过程,命令文本或其它)</param>
        /// <param name="commandText">存储过程名或SQL语句</param>
        /// <param name="commandParameters">分配给命令的DbParamter参数数组(无参数请写(DbParameter[])null)</param>
        /// <returns>返回包含结果集的IDataReader</returns>
        IDataReader ExecuteReader(CommandType commandType, string commandText, params IDataParameter[] commandParameters);

        /// <summary>
        /// 执行指定数据库连接对象的命令,指定参数,返回结果集中的第一行第一列.
        /// </summary>
        /// <remarks>
        /// 示例: int orderCount = (int)ExecuteScalar(CommandType.StoredProcedure, "GetOrderCount", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="commandType">命令类型 (存储过程,命令文本或其它)</param>
        /// <param name="commandText">存储过程名称或SQL语句</param>
        /// <param name="commandParameters">分配给命令的DbParamter参数数组(无参数请写(DbParameter[])null)</param>
        /// <returns>返回结果集中的第一行第一列</returns>
        object ExecuteScalar(CommandType commandType, string commandText, params IDataParameter[] commandParameters);

        /// <summary>
        /// 执行指定数据库连接字符串的命令,指定参数值.返回DataSet.
        /// </summary>
        /// <remarks>
        /// 示例: DataSet ds = ExecuteDataset(CommandType.StoredProcedure, "GetOrders", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="commandType">命令类型 (存储过程,命令文本或其它)</param>
        /// <param name="commandText">存储过程名称或SQL语句</param>
        /// <param name="commandParameters">分配给命令的DbParamter参数数组(无参数请写(DbParameter[])null)</param>
        /// <returns>返回一个包含结果集的DataSet</returns>
        DataSet ExecuteDataset(CommandType commandType, string commandText, params IDataParameter[] commandParameters);

        /// <summary>
        /// Command对象的传入参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">数据类型</param>
        /// <param name="size">数据长度</param>
        /// <param name="value">值</param>
        /// <returns></returns>
        IDataParameter CreateInParameter(string paramName, DbType dbType, int size, object value);
        /// <summary>
        /// Command对象的传入参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">数据类型</param>
        /// <param name="value">值</param>
        /// <returns></returns>
        IDataParameter CreateInParameter(string paramName, DbType dbType, object value);
        /// <summary>
        /// Command对象的传出参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">数据类型</param>
        /// <param name="size">数据长度</param>
        /// <returns></returns>
        IDataParameter CreateOutParameter(string paramName, DbType dbType, int size);
        /// <summary>
        /// Command对象的传出参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">数据类型</param>
        /// <returns></returns>
        IDataParameter CreateOutParameter(string paramName, DbType dbType);

        IDbTransaction BeginTransaction();

        IDbConnection DbConnection { get; }

        IDbTransaction DbTransaction { get; }

        IDatabase Table(string table);

        IDatabase Data(params SqlData[] datas);

        IDatabase Distinct();

        IDatabase Field(params SqlField[] fields);

        IDatabase Join(params SqlJoin[] joins);

        IDatabase Where(params SqlWhere[] conditions);

        IDatabase GroupBy();

        IDatabase Having(params SqlHaving[] conditions);

        IDatabase OrderBy(params SqlOrder[] orders);

        IDatabase Limit(int page, int rows);

        IDictionary Find();

        IList FindAll();

        IList FindAll(out int total);

        bool Delete();

        bool Save();

        bool Save(out int id);

        bool Update();
    }
}
