﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Data.SqlClient;
using System.Text;

namespace ThinkNet.Database
{
    /// <summary>
    /// MS-SQL
    /// </summary>
    public class SqlServerDatabase : Database
    {
        /// <summary>
        /// 通过有效的数据连接字符串构造实例
        /// </summary>
        /// <param name="connectionString"></param>
        public SqlServerDatabase(string connectionString)
            : base(SqlClientFactory.Instance, connectionString)
        { }

        public SqlServerDatabase(IDbConnection connection) :
            base(connection)
        {
            if (!(connection is SqlConnection)) {
                throw new DatabaseException("此数据连接非SqlConnection");
            }
        }

        /// <summary>
        /// 从在 <see cref="SqlCommand"/> 中指定的存储过程中检索参数信息并填充指定的 <see cref="SqlCommand"/> 对象的 <see cref="SqlCommand.Parameters"/> 集合
        /// </summary>
        /// <param name="discoveryCommand"></param>
        protected override void DeriveParameters(DbCommand discoveryCommand)
        {
            if (discoveryCommand != null) {
                SqlCommandBuilder.DeriveParameters(discoveryCommand as SqlCommand);
            }
        }

        protected override string BuildParameterName(string name)
        {
            return string.Concat("@", name);// string.Format("@{0}", name);
        }

        /// <summary>
        /// 获取自动增量的Sql
        /// </summary>
        protected override string GetIdentitySql()
        {
            return "SELECT SCOPE_IDENTITY()";
        }

        /// <summary>
        /// 获取数据库表的字段信息
        /// </summary>
        /// <param name="tableName">表名称</param>
        /// <returns></returns>
        protected override DbColumn[] GetTableColumns(string tableName)
        {
            StringBuilder sql = new StringBuilder()
                .Append("SELECT c.name, t.name, c.is_nullable, c.is_identity, ISNULL(pk.PrimaryKey,0), ISNULL(p.[value],N'') ")
                .Append("FROM sys.columns c INNER JOIN sys.objects o ON c.[object_id]=o.[object_id] AND o.type='U' AND o.is_ms_shipped=0 ")
                .Append("INNER JOIN sys.types t ON c.user_type_id=t.user_type_id ")
                .Append("LEFT JOIN sys.extended_properties p ON p.class=1 AND c.[object_id]=p.major_id AND c.column_id=p.minor_id ")
                .Append("LEFT JOIN (")
                .Append("SELECT IDXC.[object_id],IDXC.column_id, PrimaryKey=IDX.is_primary_key ")
                .Append("FROM sys.indexes IDX INNER JOIN sys.index_columns IDXC ON IDX.[object_id]=IDXC.[object_id] AND IDX.index_id=IDXC.index_id ")
                .Append(") pk ON c.[object_id]=pk.[object_id] AND c.column_id=pk.column_id ")
                .AppendFormat("WHERE o.name=N'{0}' ORDER BY o.name,c.column_id", tableName);

            List<DbColumn> arrField = new List<DbColumn>();

            IDataReader reader = base.ExecuteReader(CommandType.Text, sql.ToString(), (IDataParameter[])null);
            while (reader.Read()) {
                DbColumn field = new DbColumn();
                field.ColumnName = reader.GetString(0);
                field.DataType = reader.GetString(1);
                field.AllowDBNull = reader.GetBoolean(2);
                field.AutoIncrement = reader.GetBoolean(3);
                field.Primary = reader.GetBoolean(4);

                arrField.Add(field);
            }
            reader.Close();

            return arrField.ToArray();
        }

        protected override DbParameter CreateParameter(string paramName, DbType dbType, ParameterDirection direction, int size, object value)
        {
            //DbParameter param = _dbProviderFactory.CreateParameter();
            //param.ParameterName = BuildParameterName(paramName);

            //if (dbType != DbType.Object) {
            //    param.DbType = dbType;
            //}
            //param.Direction = direction;
            //if (size > 0)
            //    param.Size = size;
            //if (!(direction == ParameterDirection.Output && value == null))
            //    param.Value = value;

            //return param;
            return CreateSqlParameter(paramName, (SqlDbType)dbType, direction, size, value);
        }

        public override DataSet ExecuteDataset(CommandType commandType, string commandText, params IDataParameter[] commandParameters)
        {
            commandText.MustNotEmpty("commandText");

            using (IDbCommand command = CreateCommandByCommandType(commandType, commandText, commandParameters)) {
                try {
                    using (SqlDataAdapter da = new SqlDataAdapter()) {
                        //da.MissingSchemaAction = MissingSchemaAction.AddWithKey;
                        da.SelectCommand = command as SqlCommand;
                        DataSet ds = new DataSet();

                        // Fill the DataSet using default values for DataTable names, etc
                        da.Fill(ds);

                        // Return the dataset
                        return ds;
                    }
                }
                catch (Exception ex) {
                    throw ex;
                }
                finally {
                    command.Parameters.Clear();
                }
            }
        }

        protected override string BuildLikeParameterName(string parameterName)
        {
            return string.Concat("('%'+", parameterName, "+'%')");
        }

        protected override string PagingSupport(int pageIndex, int pageSize)
        {
            if (pageIndex > 0) {
                return string.Format("SELECT * FROM (SELECT ROW_NUMBER() OVER (ORDER BY Sys_RowIndex) AS Sys_RowNum,temp1.* FROM ({0}) AS temp1) AS temp2 WHERE Sys_RowNum BETWEEN {1} AND {2}",
                    "SELECT%DISTINCT% TOP (100) PERCENT%FIELDS%, ROW_NUMBER() OVER (%ORDER%) AS Sys_RowIndex%TABLE%%JOIN%%WHERE%%GROUP%%HAVING%",
                    pageIndex * pageSize + 1, pageIndex * pageSize);
            }
            else {
                return string.Format("SELECT TOP {0} * FROM (SELECT%DISTINCT% TOP (100) PERCENT%FIELDS%%TABLE%%JOIN%%WHERE%%GROUP%%HAVING%%ORDER%) AS temp1", pageSize);
            }
        }
        #region


        /// <summary>
        /// Command对象的传入参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">数据类型</param>
        /// <param name="size">数据长度</param>
        /// <param name="value">值</param>
        /// <returns></returns>
        public static DbParameter CreateInParameter(string paramName, SqlDbType dbType, int size, object value)
        {
            return CreateSqlParameter(paramName, dbType, ParameterDirection.Input, size, value);
        }
        /// <summary>
        /// Command对象的传入参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">数据类型</param>
        /// <param name="value">值</param>
        /// <returns></returns>
        public static DbParameter CreateInParameter(string paramName, SqlDbType dbType, object value)
        {
            return CreateInParameter(paramName, dbType, -1, value);
        }
        /// <summary>
        /// Command对象的传出参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">数据类型</param>
        /// <param name="size">数据长度</param>
        /// <returns></returns>
        public static DbParameter CreateOutParameter(string paramName, SqlDbType dbType, int size)
        {
            return CreateSqlParameter(paramName, dbType, ParameterDirection.Output, size, null);
        }
        /// <summary>
        /// Command对象的传出参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">数据类型</param>
        /// <returns></returns>
        public static DbParameter CreateOutParameter(string paramName, SqlDbType dbType)
        {

            return CreateOutParameter(paramName, dbType, -1);
        }

        /// <summary>
        /// 创建Commond用的DbParameter参数
        /// </summary>
        /// <param name="paramName">参数名称</param>
        /// <param name="dbType">参数对应的数据类型枚举</param>
        /// <param name="direction">参数类型</param>
        /// <param name="size">大小</param>
        /// <param name="value">值</param>
        /// <returns></returns>
        private static DbParameter CreateSqlParameter(string paramName, SqlDbType dbType, ParameterDirection direction, int size, object value)
        {
            SqlParameter param = new SqlParameter();
            param.ParameterName = string.Concat("@", paramName);
            param.SqlDbType = dbType;
            param.Direction = direction;
            if (size > 0)
                param.Size = size;
            if (!(direction == ParameterDirection.Output && value == null))
                param.Value = value;

            return param;
        }
        #endregion
    }
}
