﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Web.Script.Serialization;
using ThinkNet.Event;
using ThinkNet.Infrastructure;


namespace ThinkNet.Domain
{
    /// <summary>
    /// 可用事件追溯还原的聚合根抽象类
    /// </summary>
    /// <typeparam name="TIdentify"></typeparam>
    [Serializable]
    public abstract class SourcedAggregateRoot<TIdentify> : Entity<TIdentify>, ISourcedAggregateRoot
    {
        protected SourcedAggregateRoot()
        { }

        protected SourcedAggregateRoot(TIdentify id)
            : base(id)
        { }

        /// <summary>
        /// 版本号
        /// </summary>
        public virtual int Version
        {
            get;
            protected internal set;
        }

        /// <summary>
        /// 发布事件
        /// </summary>
        /// <typeparam name="TEvent">要发布的事件类型</typeparam>
        /// <param name="event"></param>
        protected internal virtual void PublishEvent<TEvent>(TEvent @event)
            where TEvent : class, IEvent
        {
            @event.MustNotNull("@event");

            EventBus.Publish(@event);
        }

        /// <summary>
        /// 聚合内事件
        /// </summary>
        /// <typeparam name="TEvent">要发布的事件类型</typeparam>
        /// <param name="event"></param>
        protected internal virtual void RaiseEvent<TEvent>(TEvent @event)
            where TEvent : class, IEvent
        {
            @event.MustNotNull("@event");

            HandleEvent(@event);
            QueueEvent(@event);
        }

        private void QueueEvent(IEvent uncommittedEvent)
        {
            if (_uncommittedEvents == null) {
                _uncommittedEvents = new List<IEvent>();
            }
            _uncommittedEvents.Add(uncommittedEvent);
        }

        [NonSerialized]
        private IList<IEvent> _uncommittedEvents;
        /// <summary>
        /// 获取所有未经提交的领域事件。
        /// </summary>
        [ScriptIgnore]
        IEnumerable<IEvent> IAggregateRoot.UncommittedEvents
        {
            get { return _uncommittedEvents; }
        }

        /// <summary>
        /// 当完成领域事件向外部系统提交后，清除所有已产生的领域事件。
        /// </summary>
        void IAggregateRoot.ClearEvents()
        {
            if (_uncommittedEvents != null)
                _uncommittedEvents.Clear();
        }


        void ISourcedAggregateRoot.ReplayEventStreams(IEnumerable<EventStream> eventStreams)
        {
            ((IAggregateRoot)this).ClearEvents();

            foreach (var eventStream in eventStreams) {
                if (eventStream.Version == 1) {
                    uniqueId = eventStream.AggregateRootId;
                }
                VerifyEvent(eventStream);
                ApplyEvent(eventStream);
            }
        }

        private void HandleEvent<TEvent>(TEvent @event) where TEvent : class, IEvent
        {
            var eventHandler = this as IEventHandler<TEvent>;
            if (eventHandler != null) {
                eventHandler.Handle(@event);
            }
            else {
                var _eventHandlerProvider = ServiceLocator.GetInstance<IAggregateRootInternalHandlerProvider>();
                var handler = _eventHandlerProvider.GetInternalEventHandler(GetType(), @event.GetType());
                if (handler == null)
                    throw new ThinkNetException(string.Format(Resources.EX_AGGREGATE_UNREALIZED_EVENTHANDLER,
                        GetType().FullName, @event.GetType().FullName));

                handler(this, @event);
            }
        }

        private void VerifyEvent(EventStream eventStream)
        {
            if (eventStream.AggregateRootId != uniqueId) {
                var errorMessage = string.Format("Cannot apply event stream to aggregate root as the AggregateRootId not matched. EventStream Id:{0}, AggregateRootId:{1}; Current AggregateRootId:{2}",
                                        eventStream.ID,
                                        eventStream.AggregateRootId,
                                        uniqueId);
                throw new Exception(errorMessage);
            }

            if (eventStream.Version != this.Version + 1) {
                var errorMessage = string.Format("Cannot apply event stream to aggregate root as the version not matched. EventStream Id:{0}, Version:{1}; Current AggregateRoot Version:{2}",
                                        eventStream.ID,
                                        eventStream.Version,
                                        this.Version);
                throw new Exception(errorMessage);
            }
        }
        /// <summary>Apply all the events of the given event stream to the current aggregate root.
        /// </summary>
        /// <param name="eventStream"></param>
        private void ApplyEvent(EventStream eventStream)
        {
            foreach (var @event in eventStream.Events) {
                HandleEvent(@event);
            }
            this.Version = eventStream.Version;
        }
    }


    /// <summary>
    /// 聚合根抽象类
    /// </summary>
    /// <typeparam name="TIdentify"></typeparam>
    public abstract class AggregateRoot<TIdentify> : Entity<TIdentify>, IAggregateRoot
    {
        protected AggregateRoot()
        { }

        protected AggregateRoot(TIdentify id)
            : base(id)
        { }

        /// <summary>
        /// 验证聚合合法性
        /// </summary>
        /// <returns></returns>
        public virtual IList<ModelValidationResult> Validate()
        {
            return (from property in TypeDescriptor.GetProperties(this).Cast<PropertyDescriptor>()
                    from attribute in property.Attributes.OfType<ValidationAttribute>()
                    where !attribute.IsValid(property.GetValue(this))
                    select new ModelValidationResult {
                        MemberName = property.Name,
                        Message = attribute.FormatErrorMessage(property.DisplayName ?? string.Empty)
                    }).ToList();
        }

        /// <summary>
        /// 发布事件
        /// </summary>
        /// <typeparam name="TEvent">要发布的事件类型</typeparam>
        /// <param name="event"></param>
        protected internal virtual void PublishEvent<TEvent>(TEvent @event)
            where TEvent : class, IEvent
        {
            @event.MustNotNull("@event");

            EventBus.Publish(@event);
        }

        /// <summary>
        /// 分发事件
        /// </summary>
        /// <typeparam name="TEvent"></typeparam>
        /// <param name="event"></param>
        protected internal virtual void DispenseEvent<TEvent>(TEvent @event)
            where TEvent : class, IEvent
        {
            @event.MustNotNull("@event");

            
            if (_uncommittedEvents == null) {
                _uncommittedEvents = new List<IEvent>();
            }
            _uncommittedEvents.Add(@event);
        }

        private IList<IEvent> _uncommittedEvents;
        /// <summary>
        /// 获取所有未经提交的领域事件。
        /// </summary>
        IEnumerable<IEvent> IAggregateRoot.UncommittedEvents
        {
            get { return _uncommittedEvents; }
        }

        /// <summary>
        /// 当完成领域事件向外部系统提交后，清除所有已产生的领域事件。
        /// </summary>
        void IAggregateRoot.ClearEvents()
        {
            if (_uncommittedEvents != null)
                _uncommittedEvents.Clear();
        }
    }
}
