﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Runtime.Serialization;
using ThinkNet.Infrastructure;


namespace ThinkNet.Domain
{
    /// <summary>Default implementation of IAggregateRootTypeProvider and IAssemblyInitializer.
    /// </summary>
    public class DefaultAggregateRootTypeProvider : IAggregateRootTypeProvider, IAssemblyInitializer
    {
        private readonly IDictionary<string, Type> _mappings = new Dictionary<string, Type>();

        /// <summary>Initialize from the given assemblies.
        /// </summary>
        /// <param name="assemblies"></param>
        /// <exception cref="Exception"></exception>
        public void Initialize(IObjectContainer container, IEnumerable<Assembly> assemblies)
        {
            foreach (var aggregateRootType in assemblies.SelectMany(assembly => assembly.GetTypes().Where(IsAggregateRoot))) {
                if (!aggregateRootType.IsSerializable)
                    throw new ThinkNetException(string.Format(Resources.EX_AGGREGATE_NOTSERIALIZED, aggregateRootType.FullName));

                _mappings.Add(aggregateRootType.FullName, aggregateRootType);
            }
        }
        /// <summary>Get the aggregate root type name by the aggregate root type.
        /// </summary>
        /// <param name="aggregateRootType"></param>
        /// <returns></returns>
        public string GetAggregateRootTypeName(Type aggregateRootType)
        {
            return _mappings.Single(x => x.Value == aggregateRootType).Key;
        }
        /// <summary>Get the aggregate root type by the aggregate root type name.
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public Type GetAggregateRootType(string name)
        {
            return _mappings.ContainsKey(name) ? _mappings[name] : null;
        }
        /// <summary>Get all the aggregate root types.
        /// </summary>
        /// <returns></returns>
        public IEnumerable<Type> GetAllAggregateRootTypes()
        {
            return _mappings.Values;
        }
        
        private static bool IsAggregateRoot(Type type)
        {
            return type.IsClass && !type.IsAbstract && typeof(IAggregateRoot).IsAssignableFrom(type);
        }


        public IAggregateRoot CreateAggregateRoot(Type aggregateRootType)
        {
            if (!typeof(IAggregateRoot).IsAssignableFrom(aggregateRootType)) {
                throw new Exception(string.Format("Invalid aggregate root type {0}", aggregateRootType.FullName));
            }

            return (IAggregateRoot)FormatterServices.GetUninitializedObject(aggregateRootType);
        }
    }
}
