﻿using System;
using System.Linq;

using ThinkNet.Infrastructure;
using ThinkNet.Event;


namespace ThinkNet.Domain
{
    public class EventSourcedRepository : IRepository
    {
        private readonly IAggregateRootTypeProvider _aggregateRootTypeProvider;
        private readonly ICache _aggregateCache;

        public EventSourcedRepository(IAggregateRootTypeProvider aggregateRootTypeProvider, ICache aggregateCache)
        {
            this._aggregateRootTypeProvider = aggregateRootTypeProvider;
            this._aggregateCache = aggregateCache;
        }
        
        public virtual ISourcedAggregateRoot Get(Type type, object id)
        {
            id.MustNotNull("id");
            type.MustNotNull("type");

            var aggregate = _aggregateCache.Get(id, type);
            if (aggregate == null) {
                aggregate = GetFromStorage(type, id);
                if (aggregate != null)
                    _aggregateCache.Set(aggregate);
            }

            return aggregate;
        }

        public TAggregateRoot Get<TAggregateRoot>(object id) where TAggregateRoot : class, ISourcedAggregateRoot
        {
            return this.Get(typeof(TAggregateRoot), id) as TAggregateRoot;
        }

        public virtual void SaveOrUpdate(ISourcedAggregateRoot aggregateRoot)
        {
            _aggregateCache.Set(aggregateRoot);
        }

        public virtual void Delete(ISourcedAggregateRoot aggregateRoot)
        {
            _aggregateCache.Remove(aggregateRoot.UniqueId, aggregateRoot.GetType());

            ServiceLocator.GetInstance<ISnapshotStore>().Remove(aggregateRoot.UniqueId, aggregateRoot.GetType());
            ServiceLocator.GetInstance<IEventStore>().Remove(aggregateRoot.UniqueId, aggregateRoot.GetType());
        }


        #region Helper Methods

        /// <summary>Get aggregate root from event store.
        /// </summary>
        protected ISourcedAggregateRoot GetFromStorage(Type aggregateRootType, object aggregateRootId)
        {
            ISourcedAggregateRoot aggregateRoot;

            if (TryGetFromSnapshot(aggregateRootId, aggregateRootType, out aggregateRoot)) {
                return aggregateRoot;
            }

            var _eventStore = ServiceLocator.GetInstance<IEventStore>();

            var eventStreams = _eventStore.Query(aggregateRootId, aggregateRootType, 1, int.MaxValue);
            if (eventStreams == null || !eventStreams.Any())
                throw new AggregateRootNotFoundException(aggregateRootId, aggregateRootType);


            aggregateRoot = _aggregateRootTypeProvider.CreateAggregateRoot(aggregateRootType) as ISourcedAggregateRoot; //BuildAggregateRoot(aggregateRootType, streams);
            aggregateRoot.ReplayEventStreams(eventStreams);

            return aggregateRoot;
        }
        /// <summary>Try to get an aggregate root from snapshot store.
        /// </summary>
        protected bool TryGetFromSnapshot(object aggregateRootId, Type aggregateRootType, out ISourcedAggregateRoot aggregateRoot)
        {
            aggregateRoot = null;

            var _snapshotStore = ServiceLocator.GetInstance<ISnapshotStore>();

            var snapshot = _snapshotStore.GetLastestSnapshot(aggregateRootId, aggregateRootType);
            if (snapshot == null)
                return false;

            aggregateRoot = snapshot.Data as ISourcedAggregateRoot;

            var _eventStore = ServiceLocator.GetInstance<IEventStore>();

            var eventsAfterSnapshot = _eventStore.Query(aggregateRootId, aggregateRootType, snapshot.Version + 1, int.MaxValue);
            aggregateRoot.ReplayEventStreams(eventsAfterSnapshot);

            return true;
        }


        #endregion
    }
}
