﻿using System;
using System.Collections.Generic;
using System.Threading;
#if NET40
using System.Threading.Tasks;
using System.Collections.Concurrent;
#endif

using ThinkNet.Infrastructure;



namespace ThinkNet.Event
{
    /// <summary>
    /// 事件处理总线
    /// </summary>
    public sealed class EventBus
    {
#if NET35
        public static WaitHandle[] Publish(IEvent @event)
        {
            var eventHandlers = ServiceLocator.GetInstance<IEventHandlerProvider>().GetEventHandlers(@event.GetType());

            var waitHandles = new List<WaitHandle>();

            foreach (var handler in eventHandlers) {
                var action = new Action(() => handler.Handle(@event));

                if (handler.GetType().IsDefined(typeof(AsynchronousHandlerAttribute), false))
                    HandleAsync(waitHandles, action);
                else
                    action();
            }

            return waitHandles.Any() ? waitHandles.ToArray() : new WaitHandle[0];
        }

        private static void HandleAsync(IList<WaitHandle> waitHandles, Action action)
        {
            var waitHandle = new AutoResetEvent(false);
            ThreadPool.QueueUserWorkItem(state => {
                try {
                    action();
                }
                catch (Exception ex) {
                    ServiceLocator.GetInstance<ILogger>().Error(ex, "");
                }

                ((AutoResetEvent)state).Set();
            }, waitHandle);

            waitHandles.Add(waitHandle);
        }

        public static void Publish(IEvent @event, Action<IEvent, bool, Exception> callback)
        {
            Publish(@event, callback, null);
        }

        public static void Publish(IEvent @event, Action<IEvent, bool, Exception> callback, TimeSpan? timeout)
        {
            try {
                var waitHandles = Publish(@event);

                if (waitHandles.Any()) {
                    if (!timeout.HasValue)
                        WaitHandle.WaitAll(waitHandles);
                    else
                        WaitHandle.WaitAll(waitHandles, timeout.Value);
                }

                callback(@event, true, null);
            }
            catch (Exception ex) {
                callback(@event, false, ex);
            }
        }
#endif

#if NET40
        private static bool IsAsynchronousHandler(Type eventHandlerType, Type eventType)
        {
            return eventHandlerType.IsDefined(typeof(AsynchronousHandlerAttribute), false)
                || eventHandlerType.GetMethod("Handle", new[] { eventType }).IsDefined(typeof(AsynchronousHandlerAttribute), false);
        }

        /// <summary>
        /// 发布事件
        /// </summary>
        /// <param name="event"></param>
        /// <returns></returns>
        public static Task[] Publish(IEvent @event)
        {
            var tasks = new List<Task>();

            var eventType = @event.GetType();
            var eventHandlers = ServiceLocator.GetInstance<IEventHandlerProvider>().GetEventHandlers(eventType);

            foreach (var handler in eventHandlers) {               
                if (IsAsynchronousHandler(handler.GetType(), eventType))
                    tasks.Add(handler.HandleAsync(@event));
                else
                    handler.Handle(@event);
            }

            return tasks.ToArray();
        }

        /// <summary>
        /// 发布事件(可实现回调)
        /// </summary>
        /// <typeparam name="TEvent"></typeparam>
        /// <param name="event"></param>
        /// <param name="callback"></param>
        /// <param name="timeout"></param>
        public static void Publish<TEvent>(TEvent @event, Action<TEvent, bool, Exception> callback, TimeSpan? timeout = null)
            where TEvent : class, IEvent
        {
            try {
                var tasks = Publish(@event);

                if (tasks.Length > 0) {
                    if (timeout == null)
                        Task.WaitAll(tasks);
                    else
                        Task.WaitAll(tasks, timeout.Value);
                }

                callback(@event, true, null);
            }
            catch (Exception ex) {
                callback(@event, false, ex);
            }
        }
#endif
    }
}
