﻿using System.Collections.Generic;
using System.Linq;

namespace ThinkNet.Event
{
    /// <summary>
    /// 事件处理程序管理器
    /// </summary>
    public sealed class EventProcessorManager
    {
        private static readonly EventProcessorManager _instance = new EventProcessorManager();

        static EventProcessorManager()
        { }

        /// <summary>
        /// 事件处理管理器的实例
        /// </summary>
        public static EventProcessorManager Instance
        {
            get
            {
                return _instance;
            }
        }

        private readonly List<ICommittedEventProcessor> _eventProcessors = new List<ICommittedEventProcessor>();

        private EventProcessorManager()
        { }

        /// <summary>
        /// 是否为一个空的处理器
        /// </summary>
        /// <returns></returns>
        public bool IsEmpty()
        {
            return !_eventProcessors.Any();
        }

        /// <summary>
        /// 获取所有可用的事件队列集合。
        /// </summary>
        public IEnumerable<ICommittedEventQueue> GetEventQueues()
        {
            return _eventProcessors.Select(x => x.BindingQueue);
        }

        /// <summary>
        /// 创建事件处理器
        /// </summary>
        /// <param name="eventProcessors"></param>
        public void Create(IEnumerable<ICommittedEventProcessor> eventProcessors)
        {
            if (eventProcessors == null || !eventProcessors.Any())
                return;

            _eventProcessors.AddRange(eventProcessors);
        }

        /// <summary>
        /// 根据事件队列名称集合和线程数创建事件处理器
        /// </summary>
        /// <param name="eventQueueNames"></param>
        /// <param name="workCount"></param>
        public void CreateDefault(IEnumerable<string> eventQueueNames, int workCount)
        {
            this.Create(eventQueueNames.Select(queueName => {
                return new DefaultCommittedEventProcessor(new DefaultCommittedEventQueue(queueName), workCount);
            }).ToArray());
        }

        /// <summary>
        /// 启动事件处理器
        /// </summary>
        public void Start()
        {
            _eventProcessors.ForEach(processor => {
                processor.Initialize();
                processor.Start();
            });
        }
    }
}
