﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Reflection;
using System.Timers;

using ThinkNet.Infrastructure;
using ThinkNet.Messaging;


namespace ThinkNet.Event
{

    public class DefaultCommittedEventExecutor : MessageExecutor<EventStream, ICommittedEventQueue>, ICommittedEventExecutor
    {
        private readonly IEventHandlerProvider _eventHandlerProvider;
        private readonly IEventHandleInfoStore _eventHandleInfoStore;
        private readonly ILogger _logger;
        //private readonly Timer _timer;
        private readonly ConcurrentQueue<EventStream> _retryEventQueue = new ConcurrentQueue<EventStream>();

        private const long DefaultPeriod = 5000;

        public DefaultCommittedEventExecutor(IEventHandlerProvider eventHandlerProvider, IEventHandleInfoStore eventHandleInfoStore, ILogger logger)
        {
            this._eventHandlerProvider = eventHandlerProvider;
            this._eventHandleInfoStore = eventHandleInfoStore;
            this._logger = logger;
            //this._timer = new Timer(DefaultPeriod);

            Initialize();
        }



        private void Initialize()
        {
            var timer = new Timer(DefaultPeriod);
            timer.Elapsed += new ElapsedEventHandler((object source, ElapsedEventArgs e) => {
                EventStream eventStream;
                if (_retryEventQueue.TryDequeue(out eventStream)) {
                    DispatchEventsToHandlers(eventStream);
                }

                (source as Timer).Start();
            });
            timer.AutoReset = false;
            timer.Start();
        }

        public override void Execute(EventStream eventStream, ICommittedEventQueue eventStreamQueue)
        {
            DispatchEventsToHandlers(eventStream);

            FinishExecution(eventStream, eventStreamQueue);
        }


        private void DispatchEventsToHandlers(EventStream stream)
        {
            bool success = true;
            foreach (var @event in stream.Events) {
                foreach (var handler in _eventHandlerProvider.GetEventHandlers(@event.GetType())) {
                    if (!DispatchEventToHandler(@event, handler)) {
                        _retryEventQueue.Enqueue(stream);
                        success = false;
                    }
                }
            }

            if (success) {
                foreach (var @event in stream.Events) {
                    _eventHandleInfoStore.RemoveEventHandleInfo(@event.ID);
                }
            }
        }

        private bool DispatchEventToHandler(IEvent @event, IEventHandler handler)
        {
            var eventHandlerTypeName = handler.GetInnerEventHandler().GetType().FullName;

            try {
                if (_eventHandleInfoStore.IsEventHandleInfoExist(@event.ID, eventHandlerTypeName))
                    return true;

                handler.Handle(@event);
                _eventHandleInfoStore.AddEventHandleInfo(@event.ID, eventHandlerTypeName);
                return true;
            }
            catch (Exception ex) {
                _logger.Error(string.Format("Exception raised when {0} handling {1}.", eventHandlerTypeName, @event.GetType().Name), ex);
                return false;
            }
            finally {
                if (handler is IDisposable) {
                    (handler as IDisposable).Dispose();
                }
            }
        }
    }
}
