﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

using ThinkNet.Database;
using ThinkNet.Domain;
using ThinkNet.Infrastructure;

namespace ThinkNet.Event
{
    public class EventStoreInDatabase : IEventStore
    {
        private readonly IDatabase _database;
        private readonly IStringSerializer _serializer;
        private readonly IEventTableNameProvider _eventTableProvider;
        private readonly IAggregateRootTypeProvider _aggregateRootTypeProvider;

        public EventStoreInDatabase(IDatabase database, IStringSerializer serializer,
            IEventTableNameProvider eventTableProvider, IAggregateRootTypeProvider aggregateRootTypeProvider)
        {
            this._database = database;
            this._serializer = serializer;
            this._eventTableProvider = eventTableProvider;
            this._aggregateRootTypeProvider = aggregateRootTypeProvider;
        }

        public void Append(EventStream stream)
        {
            var aggregateRootType = _aggregateRootTypeProvider.GetAggregateRootType(stream.AggregateRootName);
            var eventTable = _eventTableProvider.GetTable(aggregateRootType);
            var data = BuildDictionaryFromEventStream(stream);


            _database.Table(eventTable).Data(data).Save();
        }

        public void Remove(object aggregateRootId, Type aggregateRootType)
        {
            var eventTable = _eventTableProvider.GetTable(aggregateRootType);

            SqlWhere[] conditions = {
                                        new SqlWhere("AggregateRootId", aggregateRootId), 
                                        new SqlWhere("AggregateRootName", aggregateRootType.FullName)
                                    };

            _database.Table(eventTable).Where(conditions).Delete();
        }

        public bool IsEventStreamExist(object aggregateRootId, Type aggregateRootType, Guid id)
        {
            var eventTable = _eventTableProvider.GetTable(aggregateRootType);

            return _database.Table(eventTable).Where("Id", id).Count() > 0;
        }

        public IEnumerable<EventStream> Query(object aggregateRootId, Type aggregateRootType, int minStreamVersion, int maxStreamVersion)
        {
            var eventTable = _eventTableProvider.GetTable(aggregateRootType);
            var aggregateRootName = _aggregateRootTypeProvider.GetAggregateRootTypeName(aggregateRootType);

            SqlWhere[] conditions = {
                                        new SqlWhere("AggregateRootId", aggregateRootId), 
                                        new SqlWhere("AggregateRootName", aggregateRootName),
                                        new SqlWhere("Version", DbJudgeSymbol.EGT, minStreamVersion),
                                        new SqlWhere("Version", DbJudgeSymbol.EGT, maxStreamVersion)
                                    };



            var result = _database.Table(eventTable).Where(conditions).OrderBy("Version", DbOrderType.ASC).FindAll();
                //.Select(BuildEventStreamFrom).ToList();

            return ConvertFrom(result);
        }

        public IEnumerable<EventStream> QueryAll()
        {
            return _eventTableProvider.GetAllTables().SelectMany(table => {
                SqlOrder[] sorts = {
                                       new SqlOrder("AggregateRootName", DbOrderType.ASC),
                                       new SqlOrder("AggregateRootId", DbOrderType.ASC),
                                       new SqlOrder("Version", DbOrderType.ASC)
                                   };

                var result = _database.Table(table).OrderBy(sorts).FindAll();

                return ConvertFrom(result);
            }).ToList();
        }


        private IEnumerable<EventStream> ConvertFrom(IList data)
        {
            var list = new List<EventStream>();
            foreach (var item in data) {
                list.Add(BuildEventStreamFromDictionary((IDictionary)item));
            }

            return list;
        }

        private EventStream BuildEventStreamFromDictionary(IDictionary dict)
        {
            var events = _serializer.Deserialize<IEnumerable<IEvent>>((string)dict["Events"]);

            return new EventStream(new Guid((string)dict["Id"]),
                (string)dict["AggregateRootId"],
                (string)dict["AggregateRootName"],
                (int)dict["Version"],
                (DateTime)dict["Timestamp"],
                events);
        }

        private IDictionary BuildDictionaryFromEventStream(EventStream eventStream)
        {
            Hashtable ht = new Hashtable();
            ht["Id"] = eventStream.ID;
            ht["AggregateRootId"] = eventStream.AggregateRootId;
            ht["AggregateRootName"] = eventStream.AggregateRootName;
            ht["Version"] = eventStream.Version;
            ht["Timestamp"] = eventStream.TimeStamp;
            ht["Events"] = _serializer.Serialize(eventStream.Events);

            return ht;
        }
    }
}
