﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using ThinkNet.Domain;

namespace ThinkNet.Event
{
    /// <summary>In-memory concurrent dictionary based event store implementation.
    /// </summary>
    public class EventStoreInMemory : IEventStore
    {
        private readonly Dictionary<EventKey, EventStream> _eventDict;
        private readonly IAggregateRootTypeProvider _aggregateRootTypeProvider;

        public EventStoreInMemory(IAggregateRootTypeProvider aggregateRootTypeProvider)
        {
            this._aggregateRootTypeProvider = aggregateRootTypeProvider;
            this._eventDict = new Dictionary<EventKey, EventStream>();
        }

        /// <summary>Append the event stream to the event store.
        /// </summary>
        /// <param name="stream"></param>
        public void Append(EventStream stream)
        {
            if (stream == null)
                return;

            lock (((ICollection)_eventDict).SyncRoot) {
                _eventDict.Add(new EventKey(stream.AggregateRootId, stream.AggregateRootName, stream.Version), stream);
            }
        }


        public void Remove(object aggregateRootId, Type aggregateRootType)
        {
            lock (((ICollection)_eventDict).SyncRoot) {
                var aggregateRootName = _aggregateRootTypeProvider.GetAggregateRootTypeName(aggregateRootType);

                foreach (var key in _eventDict.Keys
                    .Where(key => key.AggregateRootId == aggregateRootId && key.AggregateRootName == aggregateRootName))
                    _eventDict.Remove(key);
            }
        }

        /// <summary>Check whether an event stream is exist in the event store.
        /// </summary>
        /// <param name="aggregateRootId"></param>
        /// <param name="aggregateRootType"></param>
        /// <param name="minStreamVersion"></param>
        /// <param name="maxStreamVersion"></param>
        /// <returns></returns>
        public IEnumerable<EventStream> Query(object aggregateRootId, Type aggregateRootType, int minStreamVersion, int maxStreamVersion)
        {
            var aggregateRootName = _aggregateRootTypeProvider.GetAggregateRootTypeName(aggregateRootType);

            return _eventDict
                .Values
                .Where(key => key.AggregateRootId == aggregateRootId && key.AggregateRootName == aggregateRootName && key.Version >= minStreamVersion && key.Version <= maxStreamVersion)
                .OrderBy(x => x.Version);
        }
        /// <summary>Query event streams from event store.
        /// </summary>
        /// <param name="aggregateRootId"></param>
        /// <param name="aggregateRootType"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        public bool IsEventStreamExist(object aggregateRootId, Type aggregateRootType, Guid id)
        {
            var aggregateRootName = _aggregateRootTypeProvider.GetAggregateRootTypeName(aggregateRootType);

            return _eventDict.Values.Any(key => key.AggregateRootId == aggregateRootId && key.AggregateRootName == aggregateRootName && key.ID == id);
        }
        /// <summary>Query all the event streams from the event store.
        /// </summary>
        /// <returns></returns>
        public IEnumerable<EventStream> QueryAll()
        {
            return _eventDict.Values;
        }

        class EventKey
        {
            public object AggregateRootId { get; private set; }
            public int Version { get; private set; }
            public string AggregateRootName { get; private set; }

            public EventKey(object aggregateRootId, string aggregateRootName, int streamVersion)
            {
                this.AggregateRootId = aggregateRootId;
                this.Version = streamVersion;
                this.AggregateRootName = aggregateRootName;
            }

            public override bool Equals(object obj)
            {
                var eventKey = obj as EventKey;

                if (eventKey == null) {
                    return false;
                }
                if (eventKey == this) {
                    return true;
                }

                return AggregateRootId == eventKey.AggregateRootId && AggregateRootName == eventKey.AggregateRootName && Version == eventKey.Version;
            }
            public override int GetHashCode()
            {
                return Utils.GetHashCode(AggregateRootId.GetHashCode(), AggregateRootName.GetHashCode(), Version.GetHashCode());
            }
        }
    }
}
