﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using ThinkNet.Database.Entity;

namespace ThinkNet.Database
{
    public static class DatabaseExtentions
    {
        private static T GetField<T>(this IDatabase db, DbAggrefunc aggregateFunc, string fieldName)
        {
            IDictionary dict = db.Field(new SqlField(aggregateFunc, fieldName, "result")).Find();
            if (dict.Count == 0)
                return default(T);


            return (T)Convert.ChangeType(dict["result"], typeof(T));
        }

        /// <summary>
        /// 获取单个字段的值
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="db"></param>
        /// <param name="field"></param>
        /// <returns></returns>
        public static T GetField<T>(this IDatabase db, string field)
        {
            return db.GetField<T>(DbAggrefunc.NONE, field);
        }

        /// <summary>
        /// 求总个数
        /// </summary>
        /// <returns></returns>
        public static int Count(this IDatabase db)
        {
            return db.GetField<int>(DbAggrefunc.COUNT, "0");
        }

        /// <summary>
        /// 求平均值
        /// </summary>
        /// <param name="field">字段名称</param>
        /// <returns></returns>
        public static T Avg<T>(this IDatabase db, string field)
        {
            return db.GetField<T>(DbAggrefunc.AVG, field);
        }

        /// <summary>
        /// 求汇总值
        /// </summary>
        /// <param name="field">字段名称</param>
        /// <returns></returns>
        public static T Sum<T>(this IDatabase db, string field)
        {
            return db.GetField<T>(DbAggrefunc.SUM, field);
        }

        /// <summary>
        /// 求最大值
        /// </summary>
        /// <param name="field">字段名称</param>
        /// <returns></returns>
        public static T Max<T>(this IDatabase db, string field)
        {
            return db.GetField<T>(DbAggrefunc.MAX, field);
        }

        /// <summary>
        /// 求最小值
        /// </summary>
        /// <param name="field">字段名称</param>
        /// <returns></returns>
        public static T Min<T>(this IDatabase db, string field)
        {
            return db.GetField<T>(DbAggrefunc.MIN, field);
        }


        /// <summary>
        /// 设置添加/修改数据
        /// </summary>
        /// <param name="data">字段与值的集合</param>
        public static IDatabase Data(this IDatabase db, IDictionary data)
        {
            if (data == null || data.Count == 0)
                return db;

            SqlData[] _data = new SqlData[data.Count];
            int index = 0;
            foreach (DictionaryEntry item in data) {
                _data[index++] = new SqlData(item.Key.ToString(), item.Value);
            }

            return db.Data(_data);
        }

        /// <summary>
        /// 连接单表
        /// </summary>
        /// <param name="joinType">连接类型</param>
        /// <param name="tableName">源表名称</param>
        /// <returns></returns>
        public static IDatabase Join(this IDatabase db, DbJoinWay joinType, string tableName, Action<SqlJoin> action)
        {
            SqlJoin join = new SqlJoin(joinType, tableName);
            action(join);

            return db.Join(join);
        }

        /// <summary>
        /// 连接单表
        /// </summary>
        /// <param name="tableName">源表名称</param>
        public static IDatabase Join(this IDatabase db, string tableName, Action<SqlJoin> action)
        {
            return db.Join(DbJoinWay.INNER, tableName, action);
        }


        public static IDatabase Field(this IDatabase db, IEnumerable<string> fields)
        {
            if (fields == null || !fields.Any())
                return db;

            var result = fields.Select(field => new SqlField(field)).ToArray();
            return db.Field(result);
        }

        /// <summary>
        /// 设置显示字段
        /// </summary>
        /// <param name="fields">字段数组</param>
        public static IDatabase Field(this IDatabase db, params string[] fields)
        {
            return db.Field(fields as IEnumerable<string>);
        }

        /// <summary>
        /// 设置显示字段
        /// </summary>
        /// <param name="fields"><see cref="SqlField"/>类的集合</param>
        public static IDatabase Field(this IDatabase db, IEnumerable<SqlField> fields)
        {
            return db.Field(fields.ToArray());
        }

        /// <summary>
        /// 设置查询条件
        /// </summary>
        /// <param name="name">字段名称</param>
        /// <param name="value">对应值</param>
        public static IDatabase Where(this IDatabase db, string name, object value)
        {
            return db.Where(name, DbJudgeSymbol.EQ, value);
        }

        /// <summary>
        /// 设置查询条件
        /// </summary>
        /// <param name="name">字段名称</param>
        /// <param name="value">对应值</param>
        public static IDatabase Where(this IDatabase db, string name, DbJudgeSymbol symbol, object value)
        {
            return db.Where(new SqlWhere(name, symbol, value));
        }

        /// <summary>
        /// 设置查询条件
        /// </summary>
        /// <param name="dict">字段与值的集合</param>
        /// <returns></returns>
        public static IDatabase Where(this IDatabase db, IDictionary<string, object> dict)
        {
            if (dict != null || dict.Count == 0)
                return db;

            var conditions = dict.Select(p => new SqlWhere(p.Key, p.Value)).ToArray();

            return db.Where(conditions);
        }

        /// <summary>
        /// 设置查询条件
        /// </summary>
        /// <param name="condition">查询类集合</param>
        public static IDatabase Where(this IDatabase db, IEnumerable<SqlWhere> conditions)
        {
            return db.Where(conditions.ToArray());
        }


        /// <summary>
        /// 设置合计函数过滤条件
        /// </summary>
        /// <param name="sqlFunc">合计函数枚举</param>
        /// <param name="name">字段名称</param>
        /// <param name="value">对应的值</param>
        /// <returns></returns>
        public static IDatabase Having(this IDatabase db, DbAggrefunc sqlFunc, string name, object value)
        {
            return db.Having(sqlFunc, name, DbJudgeSymbol.EQ, value);
        }

        public static IDatabase Having(this IDatabase db, DbAggrefunc sqlFunc, string name, DbJudgeSymbol symbol, object value)
        {
            return db.Having(new SqlHaving(sqlFunc, name, symbol, value));
        }

        /// <summary>
        /// 设置合计函数过滤条件
        /// </summary>
        /// <param name="conditions">过滤条件数组</param>
        /// <returns></returns>
        public static IDatabase Having(this IDatabase db, IEnumerable<SqlHaving> conditions)
        {
            return db.Having(conditions.ToArray());
        }

        /// <summary>
        /// 排序
        /// </summary>
        /// <param name="field">字段名称</param>
        /// <param name="orderType">排序类型</param>
        public static IDatabase OrderBy(this IDatabase db, string field, DbOrderType orderType)
        {
            return db.OrderBy(new SqlOrder(field, orderType));
        }

        /// <summary>
        /// 排序
        /// </summary>
        /// <param name="orders">排序类集合</param>
        public static IDatabase OrderBy(this IDatabase db, IEnumerable<SqlOrder> orders)
        {
            return db.OrderBy(orders.ToArray());
        }

        /// <summary>
        /// 显示TopN条数据
        /// </summary>
        /// <param name="topN"></param>
        /// <returns></returns>
        public static IDatabase Limit(this IDatabase db, int topN)
        {
            return db.Limit(1, topN);
        }

        public static bool Save<T>(this IDatabase db, T entity) where T : class
        {
            var properties = EntityUtils.GetProperties(typeof(T));
            if (properties.Where(p => p.IsDefined(typeof(AutoSequenceAttribute), false)).Count() > 1) {
                throw new DatabaseException("定义了多个自动增量属性。");
            }

            if (typeof(T).IsDefined(typeof(TableMappingAttribute), false)) {
                db.Table(typeof(T).GetAttribute<TableMappingAttribute>(false).TableName);
            }

            var mappings = EntityUtils.GetMappings(typeof(T));

            var dict = new Dictionary<string, object>();

            PropertyInfo identity = null;
            foreach (var prop in properties) {
                if (identity == null && prop.IsDefined(typeof(AutoSequenceAttribute), false)) {
                    identity = prop;
                }

                var active = prop.GetAttributes<FieldActiveAttribute>(false).Any() ? 
                    prop.GetAttribute<FieldActiveAttribute>(false).ActiveType : DbActiveType.ALL;

                if (active == DbActiveType.UPDATE)
                    continue;
                
                object value = prop.GetValue(entity, null);
                dict.Add(mappings[prop.Name], value);                
            }

            bool result;
            if (identity != null) {
                int id;
                result = db.Data(dict).Save(out id);

                if (identity != null) {
                    identity.SetValue(entity, id, null);
                }
            }
            else {
                result = db.Data(dict).Save();
            }

            return result;
        }

        public static bool Update<T>(this IDatabase db, T entity) where T : class
        {
            var properties = EntityUtils.GetProperties(typeof(T));
            var mappings = EntityUtils.GetMappings(typeof(T));

            if (typeof(T).IsDefined(typeof(TableMappingAttribute), false)) {
                db.Table(typeof(T).GetAttribute<TableMappingAttribute>(false).TableName);
            }

            var dict = new Dictionary<string, object>();

            foreach (var prop in properties) {
                var active = prop.GetAttributes<FieldActiveAttribute>(false).Any() ?
                    prop.GetAttribute<FieldActiveAttribute>(false).ActiveType : DbActiveType.ALL;

                if (active == DbActiveType.INSERT)
                    continue;

                object value = prop.GetValue(entity, null);
                dict.Add(mappings[prop.Name], value);
            }

            return db.Data(dict).Update();
        }


        /// <summary>
        /// 查询结果
        /// </summary>
        /// <typeparam name="T">实体类型</typeparam>
        /// <param name="db"></param>
        /// <returns></returns>
        public static T Find<T>(this IDatabase db) where T : class
        {
            return db.Find().ToEntity<T>();
        }
        /// <summary>
        /// 查询结果
        /// </summary>
        /// <typeparam name="T">实体类型</typeparam>
        /// <param name="db"></param>
        /// <param name="id">主键id</param>
        /// <returns></returns>
        public static T Find<T>(this Database db, object id) where T : class
        {
            return db.Find(id).ToEntity<T>();
        }

        /// <summary>
        /// 查询结果集
        /// </summary>
        /// <typeparam name="T">实体类型</typeparam>
        /// <param name="db"></param>
        /// <returns></returns>
        public static IEnumerable<T> FindAll<T>(this IDatabase db) where T : class
        {
            return db.FindAll().ToEntityList<T>();
        }
        /// <summary>
        /// 查询结果集
        /// </summary>
        /// <typeparam name="T">实体类型</typeparam>
        /// <param name="db"></param>
        /// <param name="ids">主键值的数组</param>
        /// <returns></returns>
        public static IEnumerable<T> FindAll<T>(this Database db, params object[] ids) where T : class
        {
            return db.FindAll(ids).ToEntityList<T>();
        }
        /// <summary>
        /// 查询结果集
        /// </summary>
        /// <typeparam name="T">实体类型</typeparam>
        /// <param name="db"></param>
        /// <param name="total">记录总数</param>
        /// <returns></returns>
        public static IEnumerable<T> FindAll<T>(this IDatabase db, out int total) where T : class
        {
            return db.FindAll(out total).ToEntityList<T>();
        }


        private static T ToEntity<T>(this IDictionary dict) where T : class
        {
            if (dict == null || dict.Count == 0)
                return null;


            Type entityType = typeof(T);
            T entity = (T)Activator.CreateInstance(entityType);

            var mappings = EntityUtils.GetMappings(entityType);

            PropertyInfo[] cachedProperties = EntityUtils.GetProperties(entityType);

            foreach (PropertyInfo pitem in cachedProperties) {
                string name = mappings[pitem.Name];

                if (pitem.CanWrite && dict[name] != null) {
                    pitem.SetValue(entity, dict[name], null);
                }
            }

            return entity;
        }

        private static IList<T> ToEntityList<T>(this IList list) where T : class
        {
            if (list == null || list.Count == 0)
                return null;

            List<T> result = new List<T>();
            for (int i = 0; i < list.Count; i++) {
                result.Add((list[i] as IDictionary).ToEntity<T>());
            }

            return result;
        }
    }
}
