﻿using System;
using System.Linq;


namespace System.Linq.Expressions
{
    /// <summary>
    /// Represents the extender for Expression[Func[T, bool]] type.
    /// This is part of the solution which solves
    /// the expression parameter problem when going to Entity Framework by using
    /// Apworks specifications. For more information about this solution please
    /// refer to http://blogs.msdn.com/b/meek/archive/2008/05/02/linq-to-entities-combining-predicates.aspx.
    /// </summary>
    internal static class ExpressionFuncExtender
    {
#if NET40
        private static Expression<T> Compose<T>(this Expression<T> first, Expression<T> second, Func<Expression, Expression, Expression> merge)
        {
            // build parameter map (from parameters of second to parameters of first)
            var map = first.Parameters.Select((f, i) => new { f, s = second.Parameters[i] }).ToDictionary(p => p.s, p => p.f);

            // replace parameters in the second lambda expression with parameters from the first
            var secondBody = ParameterRebinder.ReplaceParameters(map, second.Body);

            // apply composition of lambda expression bodies to parameters from the first expression 
            return Expression.Lambda<T>(merge(first.Body, secondBody), first.Parameters);
        }
#endif

        #region Public Methods
        /// <summary>
        /// Combines two given expressions by using the AND semantics.
        /// </summary>
        /// <typeparam name="T">The type of the object.</typeparam>
        /// <param name="first">The first part of the expression.</param>
        /// <param name="second">The second part of the expression.</param>
        /// <returns>The combined expression.</returns>
        public static Expression<Func<T, bool>> And<T>(this Expression<Func<T, bool>> first, Expression<Func<T, bool>> second)
        {
#if NET40
            return first.Compose(second, Expression.And);
#endif
#if NET35
            var body = Expression.And(first.Body, second.Body);
            return Expression.Lambda<Func<T, bool>>(body, first.Parameters);
#endif
        }

        /// <summary>
        /// Combines two given expressions by using the AND semantics.
        /// </summary>
        /// <typeparam name="T">The type of the object.</typeparam>
        /// <param name="first">The first part of the expression.</param>
        /// <param name="second">The second part of the expression.</param>
        /// <returns>The combined expression.</returns>
        public static Expression<Func<T, bool>> AndAlso<T>(this Expression<Func<T, bool>> first, Expression<Func<T, bool>> second)
        {
#if NET40
            return first.Compose(second, Expression.AndAlso);
#endif
#if NET35
            var body = Expression.AndAlso(first.Body, second.Body);
            return Expression.Lambda<Func<T, bool>>(body, first.Parameters);
#endif
        }

        /// <summary>
        /// Combines two given expressions by using the OR semantics.
        /// </summary>
        /// <typeparam name="T">The type of the object.</typeparam>
        /// <param name="first">The first part of the expression.</param>
        /// <param name="second">The second part of the expression.</param>
        /// <returns>The combined expression.</returns>
        public static Expression<Func<T, bool>> Or<T>(this Expression<Func<T, bool>> first, Expression<Func<T, bool>> second)
        {
#if NET40
            return first.Compose(second, Expression.Or);
#endif
#if NET35
            var body = Expression.OrElse(first.Body, second.Body);
            return Expression.Lambda<Func<T, bool>>(body, first.Parameters);
#endif
        }

        public static Expression<Func<T, bool>> Not<T>(this Expression<Func<T, bool>> expression)
        {
#if NET40
            return Expression.Lambda<Func<T, bool>>(
                Expression.Not(expression.Body),
                expression.Parameters.Single()
            );
#endif
#if NET35
            var body = Expression.Not(expression.Body);
            return Expression.Lambda<Func<T, bool>>(body, expression.Parameters);
#endif
        }
        #endregion
    }
}
