﻿using System;
using System.Collections.Generic;
using System.Linq;

using ThinkNet.Domain;


namespace ThinkNet.Infrastructure
{
    /// <summary>
    /// 应用层Service基类
    /// </summary>
    public abstract class ApplicationServiceBase : DisposableObject
    {
        private readonly IRepositoryContext _repositoryContext;
        protected internal ApplicationServiceBase()
        {
            this._repositoryContext = ServiceLocator.GetInstance<IRepositoryContext>();
        }

        protected internal ApplicationServiceBase(IRepositoryContext repositoryContext)
        {
            this._repositoryContext = repositoryContext;
        }

        /// <summary>
        /// 当前工作单元
        /// </summary>
        protected IUnitOfWork UnitOfWork
        {
            get { return this._repositoryContext; }
        }

        //public void Execute<TCommand>(TCommand command)
        //    where TCommand : class, ICommand
        //{
        //    CommandService.Current.Send(command);
        //}


        protected TDataTransferObject PerformCreateObject<TDataTransferObject, TAggregateRoot>(
            TDataTransferObject dataTransferObject,
            IRepository<TAggregateRoot> repository,
            Func<TDataTransferObject, TAggregateRoot> dtoToAggregate,
            Func<TAggregateRoot, TDataTransferObject> aggregateToDto
            )
            where TAggregateRoot : class, IAggregateRoot
        {
            return this.PerformCreateObjects(new TDataTransferObject[] { dataTransferObject }, repository, dtoToAggregate, aggregateToDto).FirstOrDefault();
        }

        protected virtual IEnumerable<TDataTransferObject> PerformCreateObjects<TDataTransferObject, TAggregateRoot>(
            IEnumerable<TDataTransferObject> dataTransferObjects,
            IRepository<TAggregateRoot> repository,
            Func<TDataTransferObject, TAggregateRoot> dtoToAggregate,
            Func<TAggregateRoot, TDataTransferObject> aggregateToDto
            )
            where TAggregateRoot : class, IAggregateRoot
        {
            dataTransferObjects.MustNotNull("dataTransferObjects");
            repository.MustNotNull("repository");

            if (dataTransferObjects.Any()) {
                List<TAggregateRoot> ars = new List<TAggregateRoot>();
 
                foreach (var dto in dataTransferObjects) {
                    var ar = dtoToAggregate(dto);

                    repository.Save(ar);

                    ars.Add(ar);
                }
                _repositoryContext.Commit();

                return ars.Select(aggregateToDto).ToList();
            }
            return new TDataTransferObject[0];
        }

        protected void PerformUpdateObject<TDataTransferObject, TAggregateRoot>(
            TDataTransferObject dataTransferObject,
            IRepository<TAggregateRoot> repository,
            Func<TDataTransferObject, object> idFieldFunc,
            Action<TAggregateRoot, TDataTransferObject> fieldUpdateAction
            ) where TAggregateRoot : class, IAggregateRoot
        {
            this.PerformUpdateObjects(new TDataTransferObject[] { dataTransferObject }, repository, idFieldFunc, fieldUpdateAction);
        }

        protected virtual void PerformUpdateObjects<TDataTransferObject, TAggregateRoot>(
            IEnumerable<TDataTransferObject> dataTransferObjects,
            IRepository<TAggregateRoot> repository,
            Func<TDataTransferObject, object> idFieldFunc,
            Action<TAggregateRoot, TDataTransferObject> fieldUpdateAction
            )
            where TAggregateRoot : class, IAggregateRoot
        {
            dataTransferObjects.MustNotNull("dataTransferObjects");
            repository.MustNotNull("repository");
            idFieldFunc.MustNotNull("idFieldFunc");
            fieldUpdateAction.MustNotNull("fieldUpdateAction");

            if (dataTransferObjects.Any()) {
                foreach (var dto in dataTransferObjects) {
                    var id = idFieldFunc(dto);
                    id.MustNotNull("ID");
                    id.ToString().MustNotEmpty("ID");
                    
                    var ar = repository.Load(id);
                    fieldUpdateAction(ar, dto);
                    repository.Update(ar);
                }
                _repositoryContext.Commit();
            }
        }

        protected void PerformDeleteObjects<TAggregateRoot, TIdentity>(
            IRepository<TAggregateRoot> repository,
            params TIdentity[] ids
            )
            where TAggregateRoot : class, IAggregateRoot
        {
            this.PerformDeleteObjects(ids as IEnumerable<TIdentity>, repository);
        }

        protected virtual void PerformDeleteObjects<TAggregateRoot, TIdentity>(
            IEnumerable<TIdentity> ids,
            IRepository<TAggregateRoot> repository
            )
            where TAggregateRoot : class, IAggregateRoot
        {
            ids.MustNotNull("ids");
            repository.MustNotNull("repository");

            if (ids.Any()) {
                foreach (var id in ids) {
                    var ar = repository.Load(id);
                    repository.Delete(ar);
                }
                _repositoryContext.Commit();
            }
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing) {
                _repositoryContext.Dispose();
            }
        }
    }
}
