﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Collections.Concurrent;


namespace ThinkNet.Infrastructure.Cache
{
    public sealed class CacheManager
    {
        private static CacheProvider _currentProvider = delegate {
            return new HashtableCacheProvider();
        };

        private static ICacheProvider cacheProvider = null;
        public static ICacheProvider CacheProvider
        {
            get
            {
                if (cacheProvider == null) {
                    cacheProvider = _currentProvider();
                }
                return cacheProvider;
            }
        }

        /// <summary>
        /// 设置缓存
        /// </summary>
        /// <param name="provider"></param>
        public static void SetProvider(CacheProvider provider)
        {
            SetProvider(provider, null);
        }

        public static void SetProvider(CacheProvider provider, IDictionary<string, string> cacheProperties)
        {
            _currentProvider = provider;
            _cacheProperties = cacheProperties;
        }

        static IDictionary<string, string> _cacheProperties = null;
#if NET40
        static readonly ConcurrentDictionary<string, ICache> _caches = new ConcurrentDictionary<string, ICache>(StringComparer.CurrentCultureIgnoreCase);
#endif
#if NET35
        static readonly IDictionary<string, ICache> _caches = new Dictionary<string, ICache>(StringComparer.CurrentCultureIgnoreCase);
#endif


        private const string defaultRegion = "ThinkNet";

        private CacheManager()
        { }



        private static ICache GetCacheByRegion(string regionName)
        {
#if NET35
            lock (((ICollection)_caches).SyncRoot) {
                if (!_caches.ContainsKey(regionName))
                    _caches.Add(regionName, CacheProvider.BuildCache(regionName, _cacheProperties));

                return _caches[regionName];
            }
#endif
#if NET40
            ICache cache;

            if (_caches.TryGetValue(regionName, out cache)) {
                return cache;
            }

            cache = CacheProvider.BuildCache(regionName, _cacheProperties);

            _caches.TryAdd(regionName, cache);

            return cache;
#endif
        }


        /// <summary>
        /// 向缓存中添加一个对象。
        /// </summary>
        /// <param name="key">缓存值的键值。</param>
        /// <param name="value">需要缓存的对象。</param>
        public static void Put(string key, object value)
        {
            Put(defaultRegion, key, value);
        }

        /// <summary>
        /// 向缓存中添加一个对象。
        /// </summary>
        /// <param name="region">缓存的区域。</param>
        /// <param name="key">缓存值的键值。</param>
        /// <param name="value">需要缓存的对象。</param>
        public static void Put(string region, string key, object value)
        {
            GetCacheByRegion(region).Put(key, value);
        }

        /// <summary>
        /// 从缓存中读取对象。
        /// </summary>
        /// <param name="key">缓存值的键值。</param>
        /// <returns>被缓存的对象。</returns>
        public static object Get(string key)
        {
            return Get(defaultRegion, key);
        }
        /// <summary>
        /// 从缓存中读取对象。
        /// </summary>
        /// <param name="region">缓存的区域。</param>
        /// <param name="key">缓存值的键值。</param>
        /// <returns>被缓存的对象。</returns>
        public static object Get(string region, string key)
        {
            return GetCacheByRegion(region).Get(key);
        }
        /// <summary>
        /// 从缓存中移除对象。
        /// </summary>
        /// <param name="key">缓存的键值。</param>
        /// <returns>被缓存的对象。</returns>
        public static void Remove(string key)
        {
            Remove(defaultRegion, key);
        }
        /// <summary>
        /// 从缓存中移除对象。
        /// </summary>
        /// <param name="region">缓存的区域。</param>
        /// <param name="key">缓存值的键值。</param>
        public static void Remove(string region, string key)
        {
            GetCacheByRegion(region).Remove(key);
        }
        /// <summary>
        /// 从缓存中移除对象。
        /// </summary>
        /// <param name="region">区域</param>
        public static void Evict(string region)
        {
            GetCacheByRegion(region).Clear();

#if NET40
            ICache cache;
            _caches.TryRemove(region, out cache);
#endif
#if NET35
            lock (((ICollection)_caches).SyncRoot) {
                if (_caches.ContainsKey(region))
                    _caches.Remove(region);
            }
#endif
        }
        ///// <summary>
        ///// 获取一个<see cref="Boolean"/>值，该值表示拥有指定键值的缓存是否存在。
        ///// </summary>
        ///// <param name="key">指定的键值。</param>
        ///// <returns>如果缓存存在，则返回true，否则返回false。</returns>
        //public bool Exists(string key)
        //{
        //    return this.Exists(region, key);
        //}
        ///// <summary>
        ///// 获取一个<see cref="Boolean"/>值，该值表示拥有指定键值和缓存值键的缓存是否存在。
        ///// </summary>
        ///// <param name="region">缓存的区域。</param>
        ///// <param name="key">缓存值的键值。</param>
        ///// <returns>如果缓存存在，则返回true，否则返回false。</returns>
        //public bool Exists(string region, string key)
        //{
        //    Cache.BuildCache(region, cacheProperties).ex
        //}
    }
}
