﻿using System;
using System.Linq;
using System.Reflection;

namespace ThinkNet.Infrastructure
{
    /// <summary>
    /// 值类型
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public abstract class ValueObject<T> : IEquatable<T>
        where T : ValueObject<T>
    {
        public bool Equals(T other)
        {
            if ((object)other == null) {
                return false;
            }

            if (Object.ReferenceEquals(this, other)) {
                return true;
            }

            var publicProperties = this.GetProperties();

            if (publicProperties.Any()) {
                return publicProperties.All(p => {
                    var left = p.GetValue(this, null);
                    var right = p.GetValue(other, null);

                    if (left is T) {
                        return Object.ReferenceEquals(left, right);
                    }

                    return Object.Equals(left, right);
                });
            }

            return true;
        }

        [NonSerialized]
        private PropertyInfo[] properties = new PropertyInfo[0];
        private PropertyInfo[] GetProperties()
        {
            if (!properties.Any()) {
                properties = this.GetType().GetProperties(BindingFlags.Public | BindingFlags.Instance | BindingFlags.DeclaredOnly);
            }

            return properties;
        }

        public override bool Equals(object obj)
        {
            if (obj == null) {
                return false;
            }

            if (Object.ReferenceEquals(this, obj)) {
                return true;
            }

            var item = obj as ValueObject<T>;

            return (object)item != null && this.Equals((T)item);
        }

        public override int GetHashCode()
        {
            int hashCode = 31;
            bool changeMultiplier = false;
            const int index = 1;

            var publicProperties = this.GetProperties();

            if (publicProperties.Any()) {
                foreach (var item in publicProperties) {
                    var value = item.GetValue(this, null);

                    if (value != null) {
                        hashCode = hashCode * ((changeMultiplier) ? 59 : 114) + value.GetHashCode();

                        changeMultiplier = !changeMultiplier;
                    }
                    else {
                        hashCode = hashCode ^ (index * 13);
                    }
                }
            }

            return hashCode;
        }

        public static bool operator ==(ValueObject<T> left, ValueObject<T> right)
        {
            if (Object.Equals(left, null)) {
                return (Object.Equals(right, null));
            }

            return left.Equals(right);
        }

        public static bool operator !=(ValueObject<T> left, ValueObject<T> right)
        {
            return !(left == right);
        }
    }
}
