﻿using System;
using System.Collections.Generic;
using System.Runtime.Remoting.Messaging;
using System.ServiceModel;
using System.Web;

using ThinkNet.TinyIoC;


namespace ThinkNet.Infrastructure
{
    class ContainerExtension : IExtension<OperationContext>
    {
        public object Value { get; set; }

        public void Attach(OperationContext owner)
        { }

        public void Detach(OperationContext owner)
        { }
    }

    class PerSessionLifetimeProvider : ITinyIoCObjectLifetimeProvider
    {
        private readonly Guid key = Guid.NewGuid();

        #region ITinyIoCObjectLifetimeProvider 成员

        public object GetObject()
        {
            object result = null;

            //Get object depending on  execution environment ( WCF without HttpContext,HttpContext or CallContext)

            if (OperationContext.Current != null) {
                //WCF without HttpContext environment
                ContainerExtension containerExtension = OperationContext.Current.Extensions.Find<ContainerExtension>();
                if (containerExtension != null) {
                    result = containerExtension.Value;
                }
            }
            else if (HttpContext.Current != null) {
                //HttpContext avaiable ( ASP.NET ..)
                if (HttpContext.Current.Items[key] != null)
                    result = HttpContext.Current.Items[key];
            }
            else {
                //Not in WCF or ASP.NET Environment, UnitTesting, WinForms, WPF etc.
                result = CallContext.GetData(key.ToString());
            }

            return result;
        }

        public void SetObject(object value)
        {
            if (OperationContext.Current != null) {
                //WCF without HttpContext environment
                ContainerExtension containerExtension = OperationContext.Current.Extensions.Find<ContainerExtension>();
                if (containerExtension == null) {
                    containerExtension = new ContainerExtension()
                    {
                        Value = value
                    };

                    OperationContext.Current.Extensions.Add(containerExtension);
                }
            }
            else if (HttpContext.Current != null) {
                //HttpContext avaiable ( ASP.NET ..)
                if (HttpContext.Current.Items[key] == null)
                    HttpContext.Current.Items[key] = value;
            }
            else {
                //Not in WCF or ASP.NET Environment, UnitTesting, WinForms, WPF etc.
                CallContext.SetData(key.ToString(), value);
            }
        }

        public void ReleaseObject()
        {
            if (OperationContext.Current != null) {
                //WCF without HttpContext environment
                ContainerExtension containerExtension = OperationContext.Current.Extensions.Find<ContainerExtension>();
                if (containerExtension != null) {
                    object obj = containerExtension.Value;
                    if (obj is IDisposable) {
                        (obj as IDisposable).Dispose();
                    }
                    OperationContext.Current.Extensions.Remove(containerExtension);
                }

            }
            else if (HttpContext.Current != null) {
                //HttpContext avaiable ( ASP.NET ..)
                if (HttpContext.Current.Items[key] != null) {
                    if (HttpContext.Current.Items[key] is IDisposable) {
                        (HttpContext.Current.Items[key] as IDisposable).Dispose();
                    }
                    HttpContext.Current.Items[key] = null;
                }
            }
            else {
                //Not in WCF or ASP.NET Environment, UnitTesting, WinForms, WPF etc.
                CallContext.FreeNamedDataSlot(key.ToString());
            }
        }

        #endregion
    }

    class PerThreadLifetimeProvider : ITinyIoCObjectLifetimeProvider
    {
        [ThreadStatic]
        private static Dictionary<Guid, object> values;
        private readonly Guid key;

        public PerThreadLifetimeProvider()
        {
            this.key = Guid.NewGuid();
        }

        #region ITinyIoCObjectLifetimeProvider 成员

        public object GetObject()
        {
            EnsureValues();

            object result;
            values.TryGetValue(this.key, out result);
            return result;

        }

        public void SetObject(object value)
        {
            EnsureValues();

            values[this.key] = value;
        }

        public void ReleaseObject()
        {
            
        }

        #endregion

        private static void EnsureValues()
        {
            // no need for locking, values is TLS
            if (values == null) {
                values = new Dictionary<Guid, object>();
            }
        }

    }

    static class TinyIoCContainerExtentions
    {
        public static TinyIoCContainer.RegisterOptions AsPerSession(this TinyIoCContainer.RegisterOptions instance)
        {
            return TinyIoCContainer.RegisterOptions.ToCustomLifetimeManager(instance, new PerSessionLifetimeProvider(), "per session singleton");
        }

        public static TinyIoCContainer.RegisterOptions AsPerThread(this TinyIoCContainer.RegisterOptions instance)
        {
            return TinyIoCContainer.RegisterOptions.ToCustomLifetimeManager(instance, new PerSessionLifetimeProvider(), "per session singleton");
        }
    }


    internal class TinyContainer : IObjectContainer
    {
        TinyIoCContainer container = new TinyIoCContainer();

        #region IObjectContainer 成员

        public bool IsRegistered(Type type)
        {
            return container.CanResolve(type);
        }

        public bool IsRegistered(Type type, string name)
        {
            return container.CanResolve(type, name, ResolveOptions.Default);
        }

        public IObjectContainer RegisterInstance(Type type, object instance, LifetimeStyle lifetimeStyle)
        {
            return this.RegisterInstance(type, instance, string.Empty, lifetimeStyle);
        }

        public IObjectContainer RegisterInstance(Type type, object instance, string name, LifetimeStyle lifetimeStyle)
        {
            switch (lifetimeStyle) {
                case LifetimeStyle.Singleton:
                    container.Register(type, instance, name);
                    break;
                case LifetimeStyle.Transient:
                    container.Register(type, instance, name).AsMultiInstance();
                    break;
                case LifetimeStyle.PerRequest:
                    container.Register(type, instance, name).AsPerSession();
                    break;
                case LifetimeStyle.PerThread:
                    container.Register(type, instance, name).AsPerThread();
                    break;
            }

            return this;
        }

        public IObjectContainer RegisterType(Type type, LifetimeStyle lifetimeStyle)
        {
            return this.RegisterType(type, string.Empty, lifetimeStyle);
        }

        public IObjectContainer RegisterType(Type type, string name, LifetimeStyle lifetimeStyle)
        {
            switch (lifetimeStyle) {
                case LifetimeStyle.Singleton:
                    container.Register(type, name).AsSingleton();
                    break;
                case LifetimeStyle.Transient:
                    container.Register(type, name).AsMultiInstance();
                    break;
                case LifetimeStyle.PerRequest:
                    container.Register(type, name).AsPerSession();
                    break;
                case LifetimeStyle.PerThread:
                    container.Register(type, name).AsPerThread();
                    break;
            }

            return this;
        }

        public IObjectContainer RegisterType(Type from, Type to, LifetimeStyle lifetimeStyle)
        {
            return this.RegisterType(from, to, string.Empty, lifetimeStyle);
        }

        public IObjectContainer RegisterType(Type from, Type to, string name, LifetimeStyle lifetimeStyle)
        {
            switch (lifetimeStyle) {
                case LifetimeStyle.Singleton:
                    container.Register(from, to, name).AsSingleton();
                    break;
                case LifetimeStyle.Transient:
                    container.Register(from, to, name).AsMultiInstance();
                    break;
                case LifetimeStyle.PerRequest:
                    container.Register(from, to, name).AsPerSession();
                    break;
                case LifetimeStyle.PerThread:
                    container.Register(from, to, name).AsPerThread();
                    break;
            }

            return this;
        }


        public object Resolve(Type type)
        {
            return container.Resolve(type);
        }

        public object Resolve(Type type, string key)
        {
            return container.Resolve(type, key);
        }

        public IEnumerable<object> ResolveAll(Type type)
        {
            return container.ResolveAll(type, true);
        }
        #endregion

        #region IDisposable 成员

        public void Dispose()
        {
            container.Dispose();
        }

        #endregion
    }
}
