﻿using System;
using System.Collections;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Mail;

namespace ThinkNet.Infrastructure.Mail
{
    public class MailClient : MailAuthentication, IMailClient
    {
        /// <summary>
        /// 构造实例
        /// </summary>
        /// <param name="key">WebConfig里section/name值</param>
        public MailClient(string key)
            : base(key)
        { }

        /// <summary>
        /// 通过指定信息构造新实例
        /// </summary>
        /// <param name="server">服务器地址</param>
        /// <param name="port">端口</param>
        /// <param name="uid">认证用户</param>
        /// <param name="pwd">验证密码</param>
        public MailClient(string server, int port, string uid, string pwd)
            : base(server, port, uid, pwd)
        { }

        /// <summary>
        /// 收件人列表
        /// </summary>
        private IList addresses = new ArrayList();

        /// <summary>
        /// 邮件附件列表
        /// </summary>
        private IList files = new ArrayList();

        protected string[] Recipients
        {
            get { return addresses.Cast<string>().ToArray(); }
        }

        protected string[] Attachments
        {
            get { return files.Cast<string>().ToArray(); }
        }

        /// <summary>
        /// 添加邮件附件
        /// </summary>
        /// <param name="attachments">附件绝对路径</param>
        protected void AddAttachment(params string[] attachments)
        {
            if (attachments == null || attachments.Length == 0)
                throw (new ArgumentNullException("attachments"));


            for (int i = 0; i < attachments.Length; i++) {
                string attachment = attachments[i].Trim();

                if (string.IsNullOrEmpty(attachment))
                    throw new ArgumentNullException("attachments[" + i + "]");

                if (!File.Exists(attachment))
                    throw new ArgumentException("文件 " + attachment + " 不存在。", "attachments[" + i + "]");

                files.Add(attachment);
            }
        }

        /// <summary>
        /// 添加一组收件人(不超过recipientmaxnum个),参数为字符串数组
        /// </summary>
        protected void AddRecipient(params string[] recipients)
        {
            if (recipients == null || recipients.Length == 0)
                throw new ArgumentNullException("recipients");

            for (int i = 0; i < recipients.Length; i++) {
                string recipient = recipients[i].Trim();

                if (string.IsNullOrEmpty(recipient))
                    throw new ArgumentNullException("recipients[" + i + "]");

                if (!recipient.IsEmail()) {
                    throw new ArgumentException("邮箱 " + recipient + " 格式不合法。", "recipients[" + i + "]");
                }

                addresses.Add(recipient);
            }
        }

        /// <summary>
        /// 发件人地址
        /// </summary>
        public string From { get; set; }

        /// <summary>
        /// 发件人名称
        /// </summary>
        public string FromName { get; set; }



        /// <summary>
        /// 邮件主题
        /// </summary>
        public string Subject { get; set; }

        /// <summary>
        /// 邮件正文
        /// </summary>
        public string Body { get; set; }

        /// <summary>
        ///  是否Html格式邮件
        /// </summary>
        public bool HtmlFormat { get; set; }


        /// <summary>
        /// 错误消息
        /// </summary>
        public string Error { get; private set; }


        /// <summary>
        /// 发送
        /// </summary>
        public virtual bool Send()
        {
            MailMessage msg = new MailMessage();
            msg.From = new MailAddress(this.From, this.FromName);
            msg.To.Add(string.Join(",", this.Recipients));
            foreach (string att in this.Attachments) {
                msg.Attachments.Add(new Attachment(att));
            }
            msg.Subject = this.Subject;
            msg.Body = this.Body;
            msg.Priority = System.Net.Mail.MailPriority.Normal;
            msg.IsBodyHtml = this.HtmlFormat; //邮件形式，.Text、.Html


            SmtpClient smtp = new SmtpClient();
            smtp.Host = this.MailServer;
            smtp.Port = this.Port;
            smtp.DeliveryMethod = SmtpDeliveryMethod.Network;
            smtp.UseDefaultCredentials = false;
            smtp.Credentials = new NetworkCredential(this.UserID, this.Password);
            smtp.EnableSsl = this.EnableSSL;

            try {
                smtp.Send(msg);

                return true;
            }
            catch (SmtpException e) {
                this.Error = e.Message;
                return false;
            }
            finally {
                msg.Dispose();
            }
        }
    }
}
