﻿using System;
using System.Linq;


namespace ThinkNet.Infrastructure
{
    /// <summary>
    /// 表示该特性的类会被加入到IOC容器中
    /// </summary>
    [AttributeUsage(AttributeTargets.Class, AllowMultiple = true, Inherited = false)]
    public class ParticipateInContainerAttribute : Attribute
    {
        public ParticipateInContainerAttribute()
            : this(LifetimeStyle.Transient, new Type[0])
        { }

        public ParticipateInContainerAttribute(params Type[] registerTypes)
            : this(LifetimeStyle.Transient, registerTypes)
        { }

        public ParticipateInContainerAttribute(LifetimeStyle lifeStyle)
            : this(lifeStyle, new Type[0])
        { }

        public ParticipateInContainerAttribute(LifetimeStyle lifeStyle, params Type[] registerTypes)
        {
            this.lifetimeStyle = lifeStyle;

            if (registerTypes == null || registerTypes.Length == 0)
                return;

            //Array.ForEach(interfaceTypes, (Type interfaceType) => {
            //    if (!interfaceType.IsInterface) {
            //        throw new InvalidOperationException(string.Concat(interfaceType.FullName, " 不是接口类型"));
            //    }
            //});

            this.registerTypes = registerTypes;
        }

        private LifetimeStyle lifetimeStyle;
        public LifetimeStyle LifetimeStyle
        {
            get { return this.lifetimeStyle; }
            set { this.lifetimeStyle = value; }
        }


        private bool registerInterface = false;
        /// <summary>
        /// 是否由系统搜索成员接口并注册到容器中
        /// </summary>
        public bool RegisterInterface
        {
            get { return this.registerInterface; }
            set { this.registerInterface = value; }
        }

        //private bool registerOnce = false;
        ///// <summary>
        ///// 同一类型是否允许注册一次或多次。true为一次，false为不限
        ///// </summary>
        //public bool RegisterOnce
        //{
        //    get { return this.registerOnce; }
        //    set { this.registerOnce = value; }
        //}

        private Type[] registerTypes = new Type[0];
        /// <summary>
        /// 注册的类型。如果指定，那么设置自动搜索接口成员的功能将无效
        /// </summary>
        public Type[] RegisterTypes
        {
            get { return this.registerTypes; }
            protected internal set { this.registerTypes = value; }
        }

        private string registerName;
        public string RegisterName
        {
            get { return this.registerName; }
            set { this.registerName = value; }
        }

        protected virtual void RegisterType(IObjectContainer container, Type type, string name)
        {
            if (string.IsNullOrEmpty(name))
                container.RegisterType(type, lifetimeStyle);
            else
                container.RegisterType(type, name, lifetimeStyle);
        }

        protected virtual void RegisterType(IObjectContainer container, Type from, Type to, string name)
        {
            if (string.IsNullOrEmpty(name))
                container.RegisterType(from, to, lifetimeStyle);
            else
                container.RegisterType(from, to, name, lifetimeStyle);
        }


        /// <summary>
        /// 执行注册
        /// </summary>
        /// <param name="container"></param>
        /// <param name="implementType"></param>
        public void Register(IObjectContainer container, Type implementType)
        {
            bool checkInterface = true;
            if (registerInterface && !registerTypes.Any()) {
                registerTypes = implementType.GetInterfaces();
                checkInterface = false;
            }

            if (!registerTypes.Any()) {
                this.RegisterType(container, implementType, registerName);

                return;
            }

            if (checkInterface) {
                Array.ForEach(registerTypes, (Type interfaceType) => {
                    if (!interfaceType.IsAssignableFrom(implementType))
                        throw new ThinkNetException(Resources.EX_CONTAINER_INVALID_INTERFACE_TYPE, implementType.FullName, interfaceType.Name);
                });
            }

            foreach (var interfaceType in registerTypes) {
                if (container.IsRegistered(interfaceType, registerName)) {
                    throw new ThinkNetException(Resources.EX_CONTAINER_TYPE_REGISTERED, interfaceType.FullName);
                }

                this.RegisterType(container, interfaceType, implementType, registerName);
            }
        }
    }
}
