﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using ThinkNet.Event;


namespace ThinkNet.Infrastructure
{
    public sealed class MemoryRepositoryContext : RepositoryContext
    {
        private static Dictionary<Type, IList<IEntity>> entityCollection = new Dictionary<Type, IList<IEntity>>();


        private static void Add(IEntity entity)
        {
            Type entityType = entity.GetType();
            if (!entityCollection.Keys.Contains(entityType)) {
                entityCollection.Add(entityType, new List<IEntity>());
            }
            entityCollection[entityType].Add(entity);
        }

        private static void Remove(IEntity entity)
        {
            Type entityType = entity.GetType();
            if (entityCollection.Keys.Contains(entityType)) {
                var item = entityCollection[entityType].FirstOrDefault(p => entity.UniqueId == p.UniqueId);
                if (item != null) {
                    entityCollection[entityType].Remove(item);
                }
            }
        }

        private readonly List<IEntity> localNewCollection = new List<IEntity>();
        private readonly List<IEntity> localModifiedCollection = new List<IEntity>();
        private readonly List<IEntity> localDeletedCollection = new List<IEntity>();

        protected override void DoCommit()
        {
            lock (((ICollection)entityCollection).SyncRoot) {
                foreach (var newObj in localNewCollection) {
                    Add(newObj);
                }
                foreach (var modifiedObj in localModifiedCollection) {
                    Remove(modifiedObj);
                    Add(modifiedObj);
                }
                foreach (var delObj in localDeletedCollection) {
                    Remove(delObj);
                }
            }
        }

        public override bool Contains(IEntity entity)
        {
            return localNewCollection.Any(p => entity.UniqueId == p.UniqueId)
                || localModifiedCollection.Any(p => entity.UniqueId == p.UniqueId)
                || entityCollection[entity.GetType()].Any(p => entity.UniqueId == p.UniqueId);
        }

        protected override void DoSave(IEntity entity)
        {
            localNewCollection.Add(entity);
        }

        protected override void DoUpdate(IEntity entity)
        {
            if (localDeletedCollection.Contains(entity))
                throw new ThinkNetException(Resources.EX_REPOSITORY_OBJECT_CANNOT_UPDATE);
            if (!localModifiedCollection.Contains(entity) && !localNewCollection.Contains(entity))
                localModifiedCollection.Add(entity);
        }

        protected override void DoDelete(IEntity entity)
        {
            if (localNewCollection.Contains(entity)) {
                if (localNewCollection.Remove(entity))
                    return;
            }
            bool removedFromModified = localModifiedCollection.Remove(entity);
            if (!localDeletedCollection.Contains(entity)) {
                localDeletedCollection.Add(entity);
            }
        }

        //public override void Persist(IEntity entity)
        //{
        //    this.DoSave(entity);
        //}

        public override void Refresh(IEntity entity)
        {
            Type entityType = entity.GetType();
            if (entityCollection.Keys.Contains(entityType)) {
                if (entityCollection[entityType].Any(p => entity.UniqueId == p.UniqueId)) {
                    entity = entityCollection[entityType].First(p => entity.UniqueId == p.UniqueId);
                }
            }
        }

        public override void Detach(IEntity entity)
        {
            if (!localDeletedCollection.Contains(entity)) {
                localDeletedCollection.Add(entity);
            }
        }

        //public override TEntity Get<TEntity, TIdentify>(TIdentify id)
        //{
        //    return this.Load<TEntity>(id);
        //}

        public override TEntity Load<TEntity>(object id)
        {
            Type entityType = typeof(TEntity);

            Func<IEntity, bool> expression = entity => {
                return entity.GetType() == entityType && entity.UniqueId == id;
            };

            if (localNewCollection.Any(expression)) {
                return localNewCollection.FirstOrDefault(expression) as TEntity;
            }

            if (localModifiedCollection.Any(expression)) {
                return localNewCollection.FirstOrDefault(expression) as TEntity;
            }
            
            if (entityCollection.Keys.Contains(entityType)) {
                return entityCollection[entityType].FirstOrDefault(p => p.UniqueId == id) as TEntity;
            }
            return null;
        }

        protected internal override int DoCount<TEntity>(ICriteria<TEntity> criteria)
        {
            Type entityType = typeof(TEntity);
            if (entityCollection.Keys.Contains(entityType)) {
                if (criteria == null || criteria.Expression == null)
                    return entityCollection[entityType].Count;

                return entityCollection[entityType].Cast<TEntity>().Count(criteria.Expression.Compile());
            }

            return 0;
        }

        protected internal override bool DoExists<TEntity>(ICriteria<TEntity> criteria)
        {
            Type entityType = typeof(TEntity);
            if (entityCollection.Keys.Contains(entityType)) {
                if (criteria == null || criteria.Expression == null)
                    return true;

                return entityCollection[entityType].Cast<TEntity>().Any(criteria.Expression.Compile());
            }

            return false;
        }

        protected internal override PagedResult<TEntity> DoQuery<TEntity>(ICriteria<TEntity> criteria, ISort<TEntity> sort, int pageIndex, int pageSize, bool cache, string region)
        {
            int totals = this.DoCount(criteria);
            IEnumerable<TEntity> data = new List<TEntity>();

            Type entityType = typeof(TEntity);
            if (entityCollection.Keys.Contains(entityType)) {
                data = entityCollection[entityType].Cast<TEntity>();

                if (criteria != null && criteria.Expression != null)
                    data = data.Where(criteria.Expression.Compile());

                if (sort != null && sort.Sorters != null) {
                    foreach (var order in sort.Sorters) {
                        if (order.SortOrder == SortOrder.Ascending) {
                            data = data.OrderBy(order.Expression.Compile());
                        }
                        else if (order.SortOrder == SortOrder.Descending) {
                            data = data.OrderByDescending(order.Expression.Compile());
                        }
                    }
                }

                data = data.ToList();
            }
            return new PagedResult<TEntity>(totals, pageSize, pageIndex, data);
        }

        protected override void Dispose(bool disposing)
        {
            localDeletedCollection.Clear();
            localModifiedCollection.Clear();
            localNewCollection.Clear();
        }

        public override System.Data.IDbConnection DbConnection
        {
            get { throw new NotImplementedException(); }
        }
    }
}
