﻿using System;
using System.Collections.Generic;
using System.Linq;
using ThinkNet.Domain;



namespace ThinkNet.Infrastructure
{
    public class Repository<TAggregateRoot> : IRepository<TAggregateRoot>
        where TAggregateRoot : class, IAggregateRoot
    {
        private IRepositoryContext context;

        /// <summary>
        /// 仓储上下文
        /// </summary>
        internal IRepositoryContext Context
        {
            get
            {
                if (this.context == null) {
                    this.context = ServiceLocator.GetInstance<IRepositoryContext>();
                }
                return this.context;
            }
        }

        protected internal RepositoryContext RepositoryContext
        {
            get
            {
                if (this.Context is RepositoryContext) {
                    return this.Context as RepositoryContext;
                }
                else {
                    throw new ThinkNetException(Resources.EX_REPOSITORY_INVALID_CONTEXT_TYPE);
                }
            }
        }

        IUnitOfWork IRepository<TAggregateRoot>.UnitOfWork
        {
            get { return this.Context; }
        }

        public Repository()
        { }

        /// <summary>
        /// 通过仓储事务上下文构建仓储
        /// </summary>
        /// <param name="context"></param>
        protected internal Repository(IRepositoryContext context)
        {
            this.context = context;
        }

        /// <summary>
        /// 判断此 <paramref name="aggregateRoot"/> 是否存在
        /// </summary>
        /// <param name="aggregateRoot"></param>
        /// <returns></returns>
        public bool Contains(TAggregateRoot aggregateRoot)
        {
            return this.Context.Contains(aggregateRoot);
        }

        /// <summary>
        /// 保存
        /// </summary>
        /// <param name="aggregateRoot"></param>
        public void Save(TAggregateRoot aggregateRoot)
        {
            if (typeof(ILifecycle).IsAssignableFrom(typeof(TAggregateRoot))) {
                LifecycleVeto veto = (aggregateRoot as ILifecycle).OnSave(this.Context);
                if (veto == LifecycleVeto.Veto)
                    return;
            }

            if (typeof(IValidatable).IsAssignableFrom(typeof(TAggregateRoot))) {
                (aggregateRoot as IValidatable).Validate(this.Context);
            }


            this.Context.Save(aggregateRoot);
        }
        /// <summary>
        /// 更新
        /// </summary>
        /// <param name="aggregateRoot"></param>
        public void Update(TAggregateRoot aggregateRoot)
        {
            if (typeof(ILifecycle).IsAssignableFrom(typeof(TAggregateRoot))) {
                LifecycleVeto veto = (aggregateRoot as ILifecycle).OnUpdate(this.Context);
                if (veto == LifecycleVeto.Veto)
                    return;
            }

            if (typeof(IValidatable).IsAssignableFrom(typeof(TAggregateRoot))) {
                (aggregateRoot as IValidatable).Validate(this.Context);
            }

            this.Context.Update(aggregateRoot);
        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="aggregateRoot"></param>
        public virtual void Delete(TAggregateRoot aggregateRoot)
        {
            if (typeof(ILifecycle).IsAssignableFrom(typeof(TAggregateRoot))) {
                LifecycleVeto veto = (aggregateRoot as ILifecycle).OnDelete(this.Context);
                if (veto == LifecycleVeto.Veto)
                    return;
            }

            this.Context.Delete(aggregateRoot);
        }
        ///// <summary>
        ///// 持久化
        ///// </summary>
        ///// <param name="aggregateRoot"></param>
        //public void Persist(TAggregateRoot aggregateRoot)
        //{
        //    //if (typeof(IValidatable).IsAssignableFrom(typeof(TAggregateRoot))) {
        //    //    (aggregateRoot as IValidatable).Validate(this.Context);
        //    //}

        //    this.Context.Persist(aggregateRoot);
        //}
        /// <summary>
        /// 刷新
        /// </summary>
        /// <param name="aggregateRoot"></param>
        public void Refresh(TAggregateRoot aggregateRoot)
        {
            this.Context.Refresh(aggregateRoot);
        }
        /// <summary>
        /// 移除
        /// </summary>
        /// <param name="item"></param>
        public void Detach(TAggregateRoot aggregateRoot)
        {
            this.Context.Detach(aggregateRoot);
        }


        /// <summary>
        /// 根据标识id获得实体
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public TAggregateRoot Load<TIdentify>(TIdentify id)
        {
            return this.Context.Load<TAggregateRoot>(id);
        }

        #region
         /// <summary>
        /// 获取符合条件的记录总数
        /// </summary>
        /// <param name="criteria">查询条件</param>
        /// <returns></returns>
        protected int Count(ICriteria<TAggregateRoot> criteria)
        {
            return this.RepositoryContext.DoCount(criteria);
        }

        /// <summary>
        /// 根据查询条件是否存在相关数据
        /// </summary>
        /// <param name="criteria">查询条件</param>
        /// <returns></returns>
        protected bool Exists(ICriteria<TAggregateRoot> criteria)
        {
            return this.RepositoryContext.DoExists(criteria);
        }

        /// <summary>
        /// 获得单个实体
        /// </summary>
        /// <param name="criteria">查询条件</param>
        /// <returns></returns>
        protected TAggregateRoot Single(ICriteria<TAggregateRoot> criteria)
        {
            return this.Single(criteria, null);
        }
        /// <summary>
        /// 获得单个实体
        /// </summary>
        /// <param name="sort">排序条件</param>
        /// <returns></returns>
        protected TAggregateRoot Single(ISort<TAggregateRoot> sort)
        {
            return this.Single(null, sort);
        }

        /// <summary>
        /// 获得单个实体
        /// </summary>
        /// <param name="criteria">查询条件</param>
        /// <param name="sorte">排序条件</param>
        /// <returns></returns>
        protected TAggregateRoot Single(ICriteria<TAggregateRoot> criteria, ISort<TAggregateRoot> sort)
        {
            return this.RepositoryContext.DoQuery(criteria, sort, -1, 1, false, string.Empty).Data.FirstOrDefault();
        }

        /// <summary>
        /// 获得所有实体
        /// </summary>
        /// <returns></returns>
        protected IEnumerable<TAggregateRoot> GetAll()
        {
            return this.GetAll(-1);
        }
        /// <summary>
        /// 获得所有实体
        /// </summary>
        /// <param name="limit">限制N条</param>
        /// <returns></returns>
        protected IEnumerable<TAggregateRoot> GetAll(int limit)
        {
            return this.GetAll(limit, null, null);
        }
        /// <summary>
        /// 获得相关一组实体
        /// </summary>
        /// <param name="sort">排序条件</param>
        /// <returns></returns>
        protected IEnumerable<TAggregateRoot> GetAll(ISort<TAggregateRoot> sort)
        {
            return this.GetAll(null, sort);
        }
        /// <summary>
        /// 获得相关一组实体
        /// </summary>
        /// <param name="criteria">查询条件</param>
        /// <returns></returns>
        protected IEnumerable<TAggregateRoot> GetAll(ICriteria<TAggregateRoot> criteria)
        {
            return this.GetAll(criteria, null);
        }
        /// <summary>
        /// 获得相关一组实体
        /// </summary>
        /// <param name="criteria">查询条件</param>
        /// <param name="sort">排序条件</param>
        /// <returns></returns>
        protected IEnumerable<TAggregateRoot> GetAll(ICriteria<TAggregateRoot> criteria, ISort<TAggregateRoot> sort)
        {
            return this.GetAll(-1, criteria, sort);
        }
        /// <summary>
        /// 获得相关一组实体
        /// </summary>
        /// <param name="limit">前N条</param>
        /// <param name="sort">排序条件</param>
        /// <returns></returns>
        protected IEnumerable<TAggregateRoot> GetAll(int limit, ISort<TAggregateRoot> sort)
        {
            return this.GetAll(limit, null, null);
        }
        /// <summary>
        /// 获得相关一组实体
        /// </summary>
        /// <param name="limit">前N条</param>
        /// <param name="criteria">查询条件</param>
        /// <returns></returns>
        protected IEnumerable<TAggregateRoot> GetAll(int limit, ICriteria<TAggregateRoot> criteria)
        {
            return this.GetAll(limit, criteria, null);
        }
        /// <summary>
        /// 获得相关一组实体
        /// </summary>
        /// <param name="limit">前N条</param>
        /// <param name="criteria">查询条件</param>
        /// <param name="sort">排序条件</param>
        /// <returns></returns>
        protected IEnumerable<TAggregateRoot> GetAll(int limit, ICriteria<TAggregateRoot> criteria, ISort<TAggregateRoot> sort)
        {
            return this.RepositoryContext.DoQuery(criteria, sort, -1, limit, false, string.Empty).Data;
        }
        /// <summary>
        /// 获得相关一组实体
        /// </summary>
        /// <param name="sort">排序条件</param>
        /// <param name="pageIndex">页索引(从0开始)</param>
        /// <param name="pageSize">页显示数量</param>
        /// <returns></returns>
        protected PagedResult<TAggregateRoot> GetAll(ISort<TAggregateRoot> sort, int pageIndex, int pageSize)
        {
            return this.GetAll(null, sort, pageIndex, pageSize);
        }
        /// <summary>
        /// 获得相关一组实体
        /// </summary>
        /// <param name="criteria">查询条件</param>
        /// <param name="sort">排序条件</param>
        /// <param name="pageIndex">页索引(从0开始)</param>
        /// <param name="pageSize">页显示数量</param>
        /// <returns></returns>
        protected PagedResult<TAggregateRoot> GetAll(ICriteria<TAggregateRoot> criteria, ISort<TAggregateRoot> sort, int pageIndex, int pageSize)
        {
            if (sort == null || !sort.Sorters.Any()) {
                throw new InvalidOperationException("无效的排序。");
            }
            if (pageIndex < 0) {
                throw new InvalidOperationException("页索引数不能小于零");
            }
            if (pageSize <= 0) {
                throw new InvalidOperationException("页显示数必须大于零");
            }


            return this.RepositoryContext.DoQuery(criteria, sort, pageIndex, pageSize, false, string.Empty);
        }

        protected string CacheRegion { get; set; }

        /// <summary>
        /// 获得所有实体
        /// </summary>
        /// <returns></returns>
        protected IEnumerable<TAggregateRoot> FindAll()
        {
            return this.FindAll(-1);
        }
        /// <summary>
        /// 获得所有实体
        /// </summary>
        /// <param name="limit">限制N条</param>
        /// <returns></returns>
        protected IEnumerable<TAggregateRoot> FindAll(int limit)
        {
            return this.FindAll(limit, null, null);
        }
        /// <summary>
        /// 获得相关一组实体
        /// </summary>
        /// <param name="sort">排序条件</param>
        /// <returns></returns>
        protected IEnumerable<TAggregateRoot> FindAll(ISort<TAggregateRoot> sort)
        {
            return this.FindAll(null, sort);
        }
        /// <summary>
        /// 获得相关一组实体
        /// </summary>
        /// <param name="criteria">查询条件</param>
        /// <returns></returns>
        protected IEnumerable<TAggregateRoot> FindAll(ICriteria<TAggregateRoot> criteria)
        {
            return this.FindAll(criteria, null);
        }
        /// <summary>
        /// 获得相关一组实体
        /// </summary>
        /// <param name="criteria">查询条件</param>
        /// <param name="sort">排序条件</param>
        /// <returns></returns>
        protected IEnumerable<TAggregateRoot> FindAll(ICriteria<TAggregateRoot> criteria, ISort<TAggregateRoot> sort)
        {
            return this.FindAll(-1, criteria, sort);
        }
        /// <summary>
        /// 获得相关一组实体
        /// </summary>
        /// <param name="limit">前N条</param>
        /// <param name="sort">排序条件</param>
        /// <returns></returns>
        protected IEnumerable<TAggregateRoot> FindAll(int limit, ISort<TAggregateRoot> sort)
        {
            return this.FindAll(limit, null, null);
        }
        /// <summary>
        /// 获得相关一组实体
        /// </summary>
        /// <param name="limit">前N条</param>
        /// <param name="criteria">查询条件</param>
        /// <returns></returns>
        protected IEnumerable<TAggregateRoot> FindAll(int limit, ICriteria<TAggregateRoot> criteria)
        {
            return this.FindAll(limit, criteria, null);
        }
        /// <summary>
        /// 获得相关一组实体
        /// </summary>
        /// <param name="limit">前N条</param>
        /// <param name="criteria">查询条件</param>
        /// <param name="sort">排序条件</param>
        /// <returns></returns>
        protected IEnumerable<TAggregateRoot> FindAll(int limit, ICriteria<TAggregateRoot> criteria, ISort<TAggregateRoot> sort)
        {
            return this.RepositoryContext.DoQuery(criteria, sort, -1, limit, true, this.CacheRegion).Data;
        }
        /// <summary>
        /// 获得相关一组实体
        /// </summary>
        /// <param name="sort">排序条件</param>
        /// <param name="pageIndex">页索引(从0开始)</param>
        /// <param name="pageSize">页显示数量</param>
        /// <returns></returns>
        protected PagedResult<TAggregateRoot> FindAll(ISort<TAggregateRoot> sort, int pageIndex, int pageSize)
        {
            return this.FindAll(null, sort, pageIndex, pageSize);
        }
        /// <summary>
        /// 获得相关一组实体
        /// </summary>
        /// <param name="criteria">查询条件</param>
        /// <param name="sort">排序条件</param>
        /// <param name="pageIndex">页索引(从0开始)</param>
        /// <param name="pageSize">页显示数量</param>
        /// <returns></returns>
        protected PagedResult<TAggregateRoot> FindAll(ICriteria<TAggregateRoot> criteria, ISort<TAggregateRoot> sort, int pageIndex, int pageSize)
        {
            if (sort == null || !sort.Sorters.Any()) {
                throw new InvalidOperationException("无效的排序。");
            }
            if (pageIndex < 0) {
                throw new InvalidOperationException("页索引数不能小于零");
            }
            if (pageSize <= 0) {
                throw new InvalidOperationException("页显示数必须大于零");
            }


            return this.RepositoryContext.DoQuery(criteria, sort, pageIndex, pageSize, true, this.CacheRegion);
        }
        #endregion
    }

    ///// <summary>
    ///// 仓储
    ///// </summary>
    ///// <typeparam name="TAggregateRoot">聚合根类型</typeparam>
    ///// <typeparam name="TIdentify">标识类型</typeparam>
    //public class Repository<TAggregateRoot, TIdentify> : Repository<TAggregateRoot>, IRepository<TAggregateRoot, TIdentify>
    //    where TAggregateRoot : class, IAggregateRoot
    //{
    //    public TAggregateRoot Get(TIdentify id)
    //    {
    //        return this.Context.Get<TAggregateRoot, TIdentify>(id);
    //    }

    //    public IEnumerable<TAggregateRoot> Get(TIdentify[] ids)
    //    {
    //        Criteria<TAggregateRoot> criteria = Criteria<TAggregateRoot>.Eval(p => ids.Contains(p.Id));

    //        return base.GetAll(criteria);
    //    }
    //}
}
