﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;

using ThinkNet.Event;
using ThinkNet.Domain;


namespace ThinkNet.Infrastructure
{
    public abstract class RepositoryContext : DisposableObject, IRepositoryContext
    {
        private readonly Guid _id = Guid.NewGuid();
        private readonly Queue<IEvent> _pendingEvents = new Queue<IEvent>();

        public Guid ID
        {
            get { return this._id; }
        }

        /// <summary>
        /// 将指定的聚合根中所保存的未提交的事件收集到RepositoryContext的本地存储中。
        /// </summary>
        /// <param name="aggregateRoot">聚合根。</param>
        protected internal void DumpPendingEvents(IAggregateRoot aggregateRoot)
        {
            if (!aggregateRoot.UncommittedEvents.Any())
                return;

            foreach (var @event in aggregateRoot.UncommittedEvents)
                _pendingEvents.Enqueue(@event);
        }

        protected abstract void DoCommit();

        private bool committed = false;

        public virtual void Commit()
        {
            committed = true;

            this.DoCommit();

            while (_pendingEvents.Count > 0) {
                var @event = _pendingEvents.Dequeue();
                EventBus.Publish(@event);
            }
        }


        protected override void Dispose(bool disposing)
        {
            this.Dispose(disposing);

            if (disposing && !committed) {
                this.Commit();
            }
        }

        public abstract bool Contains(IEntity entity);

        protected abstract void DoSave(IEntity entity);
        public void Save(IEntity entity)
        {
            if (entity.GetType().IsAssignableFrom(typeof(IAggregateRoot))) {
                this.DumpPendingEvents(entity as IAggregateRoot);
            }
            this.DoSave(entity);
        }

        protected abstract void DoUpdate(IEntity entity);
        public void Update(IEntity entity)
        {
            if (entity.GetType().IsAssignableFrom(typeof(IAggregateRoot))) {
                this.DumpPendingEvents(entity as IAggregateRoot);
            }
            this.DoUpdate(entity);
        }

        protected abstract void DoDelete(IEntity entity);
        public void Delete(IEntity entity)
        {
            if (entity.GetType().IsAssignableFrom(typeof(IAggregateRoot))) {
                this.DumpPendingEvents(entity as IAggregateRoot);
            }
            this.DoDelete(entity);
        }

        //public abstract void Persist(IEntity entity);

        public abstract void Refresh(IEntity entity);

        public abstract void Detach(IEntity entity);

        //public virtual TEntity Get<TEntity, TIdentify>(TIdentify id)
        //    where TEntity : class, IEntity
        //{
        //    return this.Load<TEntity>(id);
        //}


        public abstract TEntity Load<TEntity>(object id)
            where TEntity : class, IEntity;

        #region
        protected internal abstract int DoCount<TEntity>(ICriteria<TEntity> criteria)
            where TEntity : class;

        protected internal abstract bool DoExists<TEntity>(ICriteria<TEntity> criteria)
            where TEntity : class;

        protected internal abstract PagedResult<TEntity> DoQuery<TEntity>(ICriteria<TEntity> criteria, ISort<TEntity> sort, int pageIndex, int pageSize,
            bool cache, string region) where TEntity : class;
        #endregion



        public abstract IDbConnection DbConnection { get; }

    }
}
