﻿using System;
using System.Collections.Generic;
using ThinkNet.Infrastructure;

namespace ThinkNet.Messaging
{
    /// <summary>The abstract base message processor implementation of IMessageProcessor.
    /// </summary>
    /// <typeparam name="TMessageQueue">The type of the message queue.</typeparam>
    /// <typeparam name="TMessageExecutor">The type of the message executor.</typeparam>
    /// <typeparam name="TMessage">The type of the message.</typeparam>
    public abstract class MessageProcessor<TMessageQueue, TMessage, TMessageExecutor> : IMessageProcessor<TMessageQueue, TMessage>
        where TMessageQueue : class, IMessageQueue<TMessage>
        where TMessage : class, IMessage
        where TMessageExecutor : IMessageExecutor<TMessage, TMessageQueue>
    {
        private readonly IList<Worker> _workers;
        private readonly TMessageQueue _bindingQueue;
        private readonly ILogger _logger;
        private bool _started;

        /// <summary>The binding queue of the message processor.
        /// </summary>
        public TMessageQueue BindingQueue
        {
            get { return _bindingQueue; }
        }

        /// <summary>Parameterized constructor.
        /// </summary>
        /// <param name="bindingQueue"></param>
        /// <param name="messageExecutorCount"></param>
        /// <exception cref="ArgumentNullException"></exception>
        /// <exception cref="Exception"></exception>
        protected MessageProcessor(TMessageQueue bindingQueue, int messageExecutorCount)
        {
            if (bindingQueue == null) {
                throw new ArgumentNullException("bindingQueue");
            }
            if (messageExecutorCount <= 0) {
                throw new Exception(string.Format("There must at least one message executor for {0}.", GetType().Name));
            }

            _bindingQueue = bindingQueue;
            _workers = new List<Worker>();

            var messageExecutor =  ServiceLocator.GetInstance<TMessageExecutor>();

            for (var index = 0; index < messageExecutorCount; index++) {
                _workers.Add(new Worker(() => ProcessMessage(messageExecutor)));
            }

            _logger = ServiceLocator.GetInstance<ILogger>();
            _started = false;
        }

        /// <summary>Initialize the message processor.
        /// </summary>
        public void Initialize()
        {
            _bindingQueue.Initialize();
        }
        /// <summary>Start the message processor.
        /// </summary>
        public void Start()
        {
            if (_started)
                return;

            foreach (var worker in _workers) {
                worker.Start();
            }
            _started = true;
            _logger.Info("Processor started, binding queue {0}, worker count:{1}.", _bindingQueue.Name, _workers.Count);
        }

        private void ProcessMessage(TMessageExecutor messageExecutor)
        {
            var message = _bindingQueue.Dequeue();
            if (message == null)
                return;

            try {
                messageExecutor.Execute(message, _bindingQueue);
            }
            catch (Exception ex) {
                _logger.Error(string.Format("Exception raised when handling queue message:{0}.", message), ex);
            }
            finally {
                _bindingQueue.Complete(message);
            }
        }
    }
}
